@@ -1,4 +1,5 @@
 /*
+ * Copyright (C) 2022 Beka Gozalishvili
  * Copyright (C) 2013 Reece H. Dunn
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -20,7 +21,9 @@ import android.app.Activity;
 import android.content.Context;
 import android.content.DialogInterface;
 import android.content.SharedPreferences;
+import android.os.Build;
 import android.preference.DialogPreference;
+import android.preference.PreferenceManager;
 import android.util.AttributeSet;
 import android.view.LayoutInflater;
 import android.view.View;
@@ -328,6 +331,11 @@ public class VoiceVariantPreference extends DialogPreference {
             case DialogInterface.BUTTON_POSITIVE:
                 onDataChanged();
                 if (shouldCommit()) {
+                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N)
+                    {
+                        PreferenceManager preferenceManager = getPreferenceManager();
+                        preferenceManager.setStorageDeviceProtected ();
+                    }
                     SharedPreferences.Editor editor = getEditor();
                     if (editor != null) {
                         VoiceVariant variant = variants[mCategoryIndex][mVariantIndex].getVariant();
