@@ -67,6 +67,8 @@ static const char *help_text =
     "\t   lines less than this length as end-of-clause\n"
     "-p <integer>\n"
     "\t   Pitch adjustment, 0 to 99, default is 50\n"
+    "-P <integer>\n"
+    "\t   Pitch range adjustment, 0 to 99, default is 50\n"
     "-s <integer>\n"
     "\t   Speed in approximate words per minute. The default is 175\n"
     "-v <voice name>\n"
@@ -75,10 +77,13 @@ static const char *help_text =
     "\t   Write speech to this WAV file, rather than speaking it directly\n"
     "-b\t   Input text encoding, 1=UTF8, 2=8 bit, 4=16 bit \n"
     "-m\t   Interpret SSML markup, and ignore other < > tags\n"
+    "--ssml-break=<percentage>\n"
+    "\t   Set SSML break time multiplier, default is 100\n"
     "-q\t   Quiet, don't produce any speech (may be useful with -x)\n"
     "-x\t   Write phoneme mnemonics to stdout\n"
     "-X\t   Write phonemes mnemonics and translation trace to stdout\n"
     "-z\t   No final sentence pause at the end of the text\n"
+    "-D\t   Enable deterministic random mode\n"
     "--compile=<voice name>\n"
     "\t   Compile pronunciation rules and dictionary from the current\n"
     "\t   directory. <voice name> specifies the language\n"
@@ -86,8 +91,10 @@ static const char *help_text =
     "\t   Compile pronunciation rules and dictionary from the current\n"
     "\t   directory, including line numbers for use with -X.\n"
     "\t   <voice name> specifies the language\n"
+#if USE_MBROLA
     "--compile-mbrola=<voice name>\n"
     "\t   Compile an MBROLA voice\n"
+#endif
     "--compile-intonations\n"
     "\t   Compile the intonation data\n"
     "--compile-phonemes=<phsource-dir>\n"
@@ -143,7 +150,7 @@ static void DisplayVoices(FILE *f_out, char *language)
 	const espeak_VOICE **voices;
 	espeak_VOICE voice_select;
 
-	static char genders[4] = { '-', 'M', 'F', '-' };
+	static const char genders[4] = { '-', 'M', 'F', '-' };
 
 	if ((language != NULL) && (language[0] != 0)) {
 		// display only voices for the specified language, in order of priority
@@ -201,7 +208,7 @@ static void Write4Bytes(FILE *f, int value)
 
 static int OpenWavFile(char *path, int rate)
 {
-	static unsigned char wave_hdr[44] = {
+	static const unsigned char wave_hdr[44] = {
 		'R', 'I', 'F', 'F', 0x24, 0xf0, 0xff, 0x7f, 'W', 'A', 'V', 'E', 'f', 'm', 't', ' ',
 		0x10, 0, 0, 0, 1, 0, 1, 0,  9, 0x3d, 0, 0, 0x12, 0x7a, 0, 0,
 		2, 0, 0x10, 0, 'd', 'a', 't', 'a',  0x00, 0xf0, 0xff, 0x7f
@@ -215,7 +222,7 @@ static int OpenWavFile(char *path, int rate)
 	f_wavfile = NULL;
 	if (path[0] != 0) {
 		if (strcmp(path, "stdout") == 0) {
-#ifdef PLATFORM_WINDOWS
+#ifdef _WIN32
 			// prevent Windows adding 0x0d before 0x0a bytes
 			_setmode(_fileno(stdout), _O_BINARY);
 #endif
@@ -304,7 +311,7 @@ static void PrintVersion()
 
 int main(int argc, char **argv)
 {
-	static struct option long_options[] = {
+	static const struct option long_options[] = {
 		{ "help",    no_argument,       0, 'h' },
 		{ "stdin",   no_argument,       0, 0x100 },
 		{ "compile-debug", optional_argument, 0, 0x101 },
@@ -320,10 +327,13 @@ int main(int argc, char **argv)
 		{ "version", no_argument,       0, 0x10b },
 		{ "sep",     optional_argument, 0, 0x10c },
 		{ "tie",     optional_argument, 0, 0x10d },
+#if USE_MBROLA
 		{ "compile-mbrola", optional_argument, 0, 0x10e },
+#endif
 		{ "compile-intonations", no_argument, 0, 0x10f },
 		{ "compile-phonemes", optional_argument, 0, 0x110 },
 		{ "load",    no_argument,       0, 0x111 },
+		{ "ssml-break", required_argument, 0, 0x112 },
 		{ 0, 0, 0, 0 }
 	};
 
@@ -346,6 +356,7 @@ int main(int argc, char **argv)
 	int volume = -1;
 	int speed = -1;
 	int pitch = -1;
+	int pitch_range = -1;
 	int wordgap = -1;
 	int option_capitals = -1;
 	int option_punctuation = -1;
@@ -353,6 +364,8 @@ int main(int argc, char **argv)
 	int phoneme_options = 0;
 	int option_linelength = 0;
 	int option_waveout = 0;
+	int ssml_break = -1;
+	bool deterministic = 0;
 	
 	espeak_VOICE voice_select;
 	char filename[200];
@@ -368,7 +381,7 @@ int main(int argc, char **argv)
 	option_punctlist[0] = 0;
 
 	while (true) {
-		c = getopt_long(argc, argv, "a:b:d:f:g:hk:l:mp:qs:v:w:xXz",
+		c = getopt_long(argc, argv, "a:b:Dd:f:g:hk:l:mp:P:qs:v:w:xXz",
 		                long_options, &option_index);
 
 		// Detect the end of the options.
@@ -388,6 +401,9 @@ int main(int argc, char **argv)
 		case 'd':
 			strncpy0(devicename, optarg2, sizeof(devicename));
 			break;
+		case 'D':
+			deterministic = 1;
+			break;
 		case 'h':
 			printf("\n");
 			PrintVersion();
@@ -408,6 +424,9 @@ int main(int argc, char **argv)
 		case 'p':
 			pitch = atoi(optarg2);
 			break;
+		case 'P':
+			pitch_range = atoi(optarg2);
+			break;
 		case 'q':
 			quiet = true;
 			break;
@@ -525,6 +544,7 @@ int main(int argc, char **argv)
 			if (phonemes_separator == 'z')
 				phonemes_separator = 0x200d; // ZWJ
 			break;
+#if USE_MBROLA
 		case 0x10e: // --compile-mbrola
 		{
 			espeak_ng_InitializePath(data_path);
@@ -537,6 +557,7 @@ int main(int argc, char **argv)
 			}
 			return EXIT_SUCCESS;
 		}
+#endif
 		case 0x10f: // --compile-intonations
 		{
 			espeak_ng_InitializePath(data_path);
@@ -569,6 +590,9 @@ int main(int argc, char **argv)
 		case 0x111: // --load
 			flag_load = 1;
 			break;
+		case 0x112: // --ssml-break
+			ssml_break = atoi(optarg2);
+			break;
 		default:
 			exit(0);
 		}
@@ -583,6 +607,11 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
+	if (deterministic) {
+		// Set random generator state to well-known
+		espeak_ng_SetRandSeed(1);
+	}
+
 	if (option_waveout || quiet) {
 		// writing to a file (or no output), we can use synchronous mode
 		result = espeak_ng_InitializeOutput(ENOUTPUT_MODE_SYNCHRONOUS, 0, devicename[0] ? devicename : NULL);
@@ -645,6 +674,8 @@ int main(int argc, char **argv)
 		espeak_SetParameter(espeakVOLUME, volume, 0);
 	if (pitch >= 0)
 		espeak_SetParameter(espeakPITCH, pitch, 0);
+	if (pitch_range >= 0)
+		espeak_SetParameter(espeakRANGE, pitch_range, 0);
 	if (option_capitals >= 0)
 		espeak_SetParameter(espeakCAPITALS, option_capitals, 0);
 	if (option_punctuation >= 0)
@@ -653,6 +684,8 @@ int main(int argc, char **argv)
 		espeak_SetParameter(espeakWORDGAP, wordgap, 0);
 	if (option_linelength > 0)
 		espeak_SetParameter(espeakLINELENGTH, option_linelength, 0);
+	if (ssml_break > 0)
+		espeak_SetParameter(espeakSSML_BREAK_MUL, ssml_break, 0);
 	if (option_punctuation == 2)
 		espeak_SetPunctuationList(option_punctlist);
 
