@@ -19,7 +19,7 @@
 #ifndef UNISTD_H_COMPAT_SHIM
 #define UNISTD_H_COMPAT_SHIM
 
-#if defined(HAVE_UNISTD_H)
+#if __has_include_next(<unistd.h>)
 #pragma GCC system_header // Silence "warning: #include_next is a GCC extension"
 #include_next <unistd.h>
 #endif
