@@ -30,18 +30,27 @@
 #include <espeak-ng/encoding.h>
 
 #include "phonemelist.h"
-#include "synthdata.h"
+#include "phoneme.h"     // for PHONEME_TAB, phVOWEL, REPLACE_PHONEMES, phPAUSE
+#include "synthdata.h"   // for InterpretPhoneme, SelectPhonemeTable
+#include "synthesize.h"  // for PHONEME_LIST, PHONEME_LIST2, phoneme_tab
+#include "translate.h"   // for Translator, LANGUAGE_OPTIONS, option_wordgap
 
 #include "phoneme.h"
-#include "voice.h"
 #include "synthesize.h"
 #include "translate.h"
+#include "speech.h"
 
-const unsigned char pause_phonemes[8] = {
+static void SetRegressiveVoicing(int regression, PHONEME_LIST2 *plist2, PHONEME_TAB *ph, Translator *tr);
+static void ReInterpretPhoneme(PHONEME_TAB *ph, PHONEME_TAB *ph2, PHONEME_LIST *plist3, Translator *tr, PHONEME_DATA *phdata, WORD_PH_DATA *worddata);
+
+static const unsigned char pause_phonemes[8] = {
 	0, phonPAUSE_VSHORT, phonPAUSE_SHORT, phonPAUSE, phonPAUSE_LONG, phonGLOTTALSTOP, phonPAUSE_LONG, phonPAUSE_LONG
 };
 
-static int SubstitutePhonemes(PHONEME_LIST *plist_out, int n_ph_list2, PHONEME_LIST2 *ph_list2)
+extern int n_ph_list2;
+extern PHONEME_LIST2 ph_list2[N_PHONEME_LIST]; // first stage of text->phonemes
+
+static int SubstitutePhonemes(PHONEME_LIST *plist_out)
 {
 	// Copy the phonemes list and perform any substitutions that are required for the
 	// current voice
@@ -61,13 +70,16 @@ static int SubstitutePhonemes(PHONEME_LIST *plist_out, int n_ph_list2, PHONEME_L
 			deleted_sourceix = -1;
 		}
 
+		if (plist2->phcode == phonSWITCH)
+			SelectPhonemeTable(plist2->tone_ph);
+
 		// don't do any substitution if the language has been temporarily changed
 		if (!(plist2->synthflags & SFLAG_SWITCHED_LANG)) {
 			if (ix < (n_ph_list2 -1))
 				next = phoneme_tab[ph_list2[ix+1].phcode];
 
 			word_end = false;
-			if ((plist2+1)->sourceix || ((next != 0) && (next->type == phPAUSE)))
+			if (ix == n_ph_list2 -1 || (plist2+1)->sourceix || ((next != 0) && (next->type == phPAUSE)))
 				word_end = true; // this phoneme is the end of a word
 
 			// check whether a Voice has specified that we should replace this phoneme
@@ -107,26 +119,22 @@ static int SubstitutePhonemes(PHONEME_LIST *plist_out, int n_ph_list2, PHONEME_L
 	return n_plist_out;
 }
 
-void MakePhonemeList(Translator *tr, int post_pause, bool start_sentence, int *n_ph_list2, PHONEME_LIST2 *ph_list2)
+void MakePhonemeList(Translator *tr, int post_pause, bool start_sentence)
 {
 	int ix = 0;
 	int j;
 	int insert_ph = 0;
 	PHONEME_LIST *phlist;
-	PHONEME_TAB *ph;
-	PHONEME_TAB *next, *next2;
+	PHONEME_TAB *ph = NULL;
+	PHONEME_TAB *next;
 	int unstress_count = 0;
 	int word_stress = 0;
 	int current_phoneme_tab;
 	int max_stress;
-	int voicing;
-	int regression;
 	int end_sourceix;
 	int alternative;
 	int delete_count;
 	int word_start;
-	bool inserted;
-	bool deleted;
 	PHONEME_DATA phdata;
 	bool start_of_clause = true;
 
@@ -141,11 +149,12 @@ void MakePhonemeList(Translator *tr, int post_pause, bool start_sentence, int *n
 	memset(&worddata, 0, sizeof(worddata));
 	plist2 = ph_list2;
 	phlist = phoneme_list;
-	end_sourceix = plist2[*n_ph_list2 - 1].sourceix;
+	end_sourceix = plist2[n_ph_list2-1].sourceix;
+	MAKE_MEM_UNDEFINED(&phoneme_list, sizeof(phoneme_list));
 
 	// is the last word of the clause unstressed ?
 	max_stress = 0;
-	for (j = *n_ph_list2 - 3; j >= 0; j--) {
+	for (j = n_ph_list2-3; j >= 0; j--) {
 		// start with the last phoneme (before the terminating pauses) and move backwards
 		if ((plist2[j].stresslevel & 0x7f) > max_stress)
 			max_stress = plist2[j].stresslevel & 0x7f;
@@ -170,7 +179,7 @@ void MakePhonemeList(Translator *tr, int post_pause, bool start_sentence, int *n
 	delete_count = 0;
 	current_phoneme_tab = tr->phoneme_tab_ix;
 	int deleted_sourceix = -1;
-	for (j = 0; j < *n_ph_list2; j++) {
+	for (j = 0; j < n_ph_list2; j++) {
 		if (current_phoneme_tab != tr->phoneme_tab_ix)
 			plist2[j].synthflags |= SFLAG_SWITCHED_LANG;
 
@@ -198,77 +207,17 @@ void MakePhonemeList(Translator *tr, int post_pause, bool start_sentence, int *n
 		}
 
 	}
-	*n_ph_list2 -= delete_count;
+	n_ph_list2 -= delete_count;
 
+	SelectPhonemeTable(current_phoneme_tab);
+
+	int regression;
 	if ((regression = tr->langopts.param[LOPT_REGRESSIVE_VOICING]) != 0) {
-		// set consonant clusters to all voiced or all unvoiced
-		// Regressive
-		int type;
-		bool stop_propagation = false;
-		voicing = 0;
-
-		for (j = *n_ph_list2 - 1; j >= 0; j--) {
-			ph = phoneme_tab[plist2[j].phcode];
-			if (ph == NULL)
-				continue;
-
-			if (plist2[j].synthflags & SFLAG_SWITCHED_LANG) {
-				stop_propagation = false;
-				voicing = 0;
-				if (regression & 0x100)
-					voicing = 1; // word-end devoicing
-				continue;
-			}
-
-			type = ph->type;
-
-			if (regression & 0x2) {
-				// [v] amd [v;] don't cause regression, or [R^]
-				if (((ph->mnemonic & 0xff) == 'v') || ((ph->mnemonic & 0xff) == 'R')) {
-					stop_propagation = true;
-					if (regression & 0x10)
-						voicing = 0;
-				}
-			}
-
-			if ((type == phSTOP) || type == (phFRICATIVE)) {
-				if ((voicing == 0) && (regression & 0xf))
-					voicing = 1;
-				else if ((voicing == 2) && (ph->end_type != 0)) // use end_type field for voicing_switch for consonants
-					plist2[j].phcode = ph->end_type; // change to voiced equivalent
-			} else if ((type == phVSTOP) || type == (phVFRICATIVE)) {
-				if ((voicing == 0) && (regression & 0xf))
-					voicing = 2;
-				else if ((voicing == 1) && (ph->end_type != 0))
-					plist2[j].phcode = ph->end_type; // change to unvoiced equivalent
-			} else {
-				if (regression & 0x8) {
-					// LANG=Polish, propagate through liquids and nasals
-					if ((type == phPAUSE) || (type == phVOWEL))
-						voicing = 0;
-				} else
-					voicing = 0;
-			}
-			if (stop_propagation) {
-				voicing = 0;
-				stop_propagation = false;
-			}
-
-			if (plist2[j].sourceix) {
-				if (regression & 0x04) {
-					// stop propagation at a word boundary
-					voicing = 0;
-				}
-				if (regression & 0x100) {
-					// devoice word-final consonants, unless propagating voiced
-					if (voicing == 0)
-						voicing = 1;
-				}
-			}
-		}
+		SetRegressiveVoicing(regression, plist2, ph, tr);
 	}
 
-	n_ph_list3 = SubstitutePhonemes(ph_list3, (int) *n_ph_list2, ph_list2) - 2;
+	SelectPhonemeTable(tr->phoneme_tab_ix);
+	n_ph_list3 = SubstitutePhonemes(ph_list3) - 2;
 
 	for (j = 0; (j < n_ph_list3) && (ix < N_PHONEME_LIST-3);) {
 		if (ph_list3[j].sourceix) {
@@ -298,11 +247,12 @@ void MakePhonemeList(Translator *tr, int post_pause, bool start_sentence, int *n
 	ph_list3[0].ph = ph;
 	word_start = 1;
 
+	SelectPhonemeTable(tr->phoneme_tab_ix);
 	for (j = 0; insert_ph || ((j < n_ph_list3) && (ix < N_PHONEME_LIST-3)); j++) {
 		plist3 = &ph_list3[j];
 
-		inserted = false;
-		deleted = false;
+		bool inserted = false;
+		bool deleted = false;
 		if (insert_ph != 0) {
 			// we have a (linking) phoneme which we need to insert here
 			next = phoneme_tab[plist3->phcode];      // this phoneme, i.e. after the insert
@@ -365,16 +315,7 @@ void MakePhonemeList(Translator *tr, int post_pause, bool start_sentence, int *n
 			plist3->ph = ph;
 			plist3->phcode = alternative;
 
-			if (ph->type == phVOWEL) {
-				plist3->synthflags |= SFLAG_SYLLABLE;
-				if (ph2->type != phVOWEL)
-					plist3->stresslevel = 0; // change from non-vowel to vowel, make sure it's unstressed
-			} else
-				plist3->synthflags &= ~SFLAG_SYLLABLE;
-
-			// re-interpret the changed phoneme
-			// But it doesn't obey a second ChangePhoneme()
-			InterpretPhoneme(tr, 0x100, plist3, &phdata, &worddata);
+			ReInterpretPhoneme(ph, ph2, plist3, tr, &phdata, &worddata);
 		}
 
 		if ((alternative = phdata.pd_param[pd_CHANGEPHONEME]) > 0) {
@@ -387,22 +328,11 @@ void MakePhonemeList(Translator *tr, int post_pause, bool start_sentence, int *n
 			if (alternative == 1)
 				deleted = true; // NULL phoneme, discard
 			else {
-				if (ph->type == phVOWEL) {
-					plist3->synthflags |= SFLAG_SYLLABLE;
-					if (ph2->type != phVOWEL)
-						plist3->stresslevel = 0; // change from non-vowel to vowel, make sure it's unstressed
-				} else
-					plist3->synthflags &= ~SFLAG_SYLLABLE;
-
-				// re-interpret the changed phoneme
-				// But it doesn't obey a second ChangePhoneme()
-				InterpretPhoneme(tr, 0x100, plist3, &phdata, &worddata);
+				ReInterpretPhoneme(ph, ph2, plist3, tr, &phdata, &worddata);
 			}
 		}
 
 		if ((ph->type == phVOWEL) && (deleted == false)) {
-			PHONEME_LIST *p;
-
 			// Check for consecutive unstressed syllables, even across word boundaries.
 			// Do this after changing phonemes according to stress level.
 			if (plist3->stresslevel <= 1) {
@@ -411,6 +341,7 @@ void MakePhonemeList(Translator *tr, int post_pause, bool start_sentence, int *n
 
 				if (tr->langopts.stress_flags & 0x08) {
 					// change sequences of consecutive unstressed vowels in unstressed words to diminished stress (TEST)
+					PHONEME_LIST *p;
 					for (p = plist3+1; p->type != phPAUSE; p++) {
 						if (p->type == phVOWEL) {
 							if (p->stresslevel <= 1) {
@@ -438,7 +369,7 @@ void MakePhonemeList(Translator *tr, int post_pause, bool start_sentence, int *n
 		}
 
 		if ((plist3+1)->synthflags & SFLAG_LENGTHEN) {
-			static char types_double[] = { phFRICATIVE, phVFRICATIVE, phNASAL, phLIQUID, 0 };
+			static const char types_double[] = { phFRICATIVE, phVFRICATIVE, phNASAL, phLIQUID, 0 };
 			if ((j > 0) && (strchr(types_double, next->type))) {
 				// lengthen this consonant by doubling it
 				// BUT, can't insert a phoneme at position plist3[0] because it crashes PrevPh()
@@ -493,8 +424,7 @@ void MakePhonemeList(Translator *tr, int post_pause, bool start_sentence, int *n
 			}
 		}
 
-		next2 = phoneme_tab[plist3[2].phcode];
-		plist3[2].ph = next2;
+		plist3[2].ph = phoneme_tab[plist3[2].phcode];
 
 		if ((insert_ph == 0) && (phdata.pd_param[pd_APPENDPHONEME] != 0))
 			insert_ph = phdata.pd_param[pd_APPENDPHONEME];
@@ -552,14 +482,112 @@ void MakePhonemeList(Translator *tr, int post_pause, bool start_sentence, int *n
 	phlist[ix].length = post_pause; // length of the pause, depends on the punctuation
 	phlist[ix].sourceix = end_sourceix;
 	phlist[ix].synthflags = 0;
+	phlist[ix].prepause = 0;
 	phlist[ix++].ph = phoneme_tab[phonPAUSE];
 
+	phlist[ix].newword = 0;
+
 	phlist[ix].phcode = phonPAUSE;
 	phlist[ix].type = phPAUSE;
 	phlist[ix].length = 0;
 	phlist[ix].sourceix = 0;
 	phlist[ix].synthflags = 0;
+	phlist[ix].prepause = 0;
 	phlist[ix++].ph = phoneme_tab[phonPAUSE_SHORT];
 
 	n_phoneme_list = ix;
+
+	SelectPhonemeTable(tr->phoneme_tab_ix);
+}
+
+static void SetRegressiveVoicing(int regression, PHONEME_LIST2 *plist2, PHONEME_TAB *ph, Translator *tr) {
+		// set consonant clusters to all voiced or all unvoiced
+		// Regressive
+		int type;
+		bool stop_propagation = false;
+		int voicing = 0;
+
+		for (int j = n_ph_list2-1; j >= 0; j--) {
+			if (plist2[j].phcode == phonSWITCH) {
+				/* Find previous phonSWITCH to determine language we're switching back to */
+				int k;
+				for (k = j-1; k >= 0; k--)
+					if (plist2[k].phcode == phonSWITCH)
+						break;
+				if (k >= 0)
+					SelectPhonemeTable(plist2[k].tone_ph);
+				else
+					SelectPhonemeTable(tr->phoneme_tab_ix);
+			}
+			ph = phoneme_tab[plist2[j].phcode];
+			if (ph == NULL)
+				continue;
+
+			if (plist2[j].synthflags & SFLAG_SWITCHED_LANG) {
+				stop_propagation = false;
+				voicing = 0;
+				if (regression & 0x100)
+					voicing = 1; // word-end devoicing
+				continue;
+			}
+
+			type = ph->type;
+
+			if (regression & 0x2) {
+				// [v] amd [v;] don't cause regression, or [R^]
+				if (((ph->mnemonic & 0xff) == 'v') || ((ph->mnemonic & 0xff) == 'R')) {
+					stop_propagation = true;
+					if (regression & 0x10)
+						voicing = 0;
+				}
+			}
+
+			if ((type == phSTOP) || type == (phFRICATIVE)) {
+				if ((voicing == 0) && (regression & 0xf))
+					voicing = 1;
+				else if ((voicing == 2) && (ph->end_type != 0)) // use end_type field for voicing_switch for consonants
+					plist2[j].phcode = ph->end_type; // change to voiced equivalent
+			} else if ((type == phVSTOP) || type == (phVFRICATIVE)) {
+				if ((voicing == 0) && (regression & 0xf))
+					voicing = 2;
+				else if ((voicing == 1) && (ph->end_type != 0))
+					plist2[j].phcode = ph->end_type; // change to unvoiced equivalent
+			} else {
+				if (regression & 0x8) {
+					// LANG=Polish, propagate through liquids and nasals
+					if ((type == phPAUSE) || (type == phVOWEL))
+						voicing = 0;
+				} else
+					voicing = 0;
+			}
+			if (stop_propagation) {
+				voicing = 0;
+				stop_propagation = false;
+			}
+
+			if (plist2[j].sourceix) {
+				if (regression & 0x04) {
+					// stop propagation at a word boundary
+					voicing = 0;
+				}
+				if (regression & 0x100) {
+					// devoice word-final consonants, unless propagating voiced
+					if (voicing == 0)
+						voicing = 1;
+				}
+			}
+		}
+	}
+
+static void ReInterpretPhoneme(PHONEME_TAB *ph, PHONEME_TAB *ph2, PHONEME_LIST *plist3, Translator *tr, PHONEME_DATA *phdata, WORD_PH_DATA *worddata) {
+if (ph->type == phVOWEL) {
+				plist3->synthflags |= SFLAG_SYLLABLE;
+				if (ph2->type != phVOWEL)
+					plist3->stresslevel = 0; // change from non-vowel to vowel, make sure it's unstressed
+			} else
+				plist3->synthflags &= ~SFLAG_SYLLABLE;
+
+			// re-interpret the changed phoneme
+			// But it doesn't obey a second ChangePhoneme()
+			InterpretPhoneme(tr, 0x100, plist3, phdata, worddata);
 }
