@@ -37,19 +37,22 @@
 #include <ucd/ucd.h>
 
 #include "readclause.h"
-#include "config.h"               // for HAVE_MKSTEMP
+#include "common.h"               // for GetFileLength, strncpy0
 #include "dictionary.h"           // for LookupDictList, DecodePhonemes, Set...
 #include "error.h"                // for create_file_error_context
 #include "phoneme.h"              // for phonSWITCH
 #include "soundicon.h"               // for LookupSoundIcon
-#include "speech.h"               // for GetFileLength, LookupMnem, PATHSEP
+#include "speech.h"               // for LookupMnem, PATHSEP
 #include "ssml.h"                 // for SSML_STACK, ProcessSsmlTag, N_PARAM...
 #include "synthdata.h"            // for SelectPhonemeTable
 #include "translate.h"            // for Translator, utf8_out, CLAUSE_OPTION...
-#include "voice.h"                // for voice, voice_t, current_voice_selected
+#include "voice.h"                // for voice, voice_t, espeak_GetCurrentVoice
 
 #define N_XML_BUF   500
 
+static void DecodeWithPhonemeMode(char *buf, char *phonemes, Translator *tr, Translator *tr2, unsigned int flags[]);
+static void TerminateBufWithSpaceAndZero(char *buf, int index, int *ungetc);
+
 static const char *xmlbase = ""; // base URL from <speak>
 
 static int namedata_ix = 0;
@@ -130,23 +133,6 @@ int clause_type_from_codepoint(uint32_t c)
 	return CLAUSE_NONE;
 }
 
-int is_str_totally_null(const char* str, int size) {
-	// Tests if all bytes of str are null up to size
-	// This should never be reimplemented with integers, because
-	// this function has to work with unaligned char*
-	// (casting to int when unaligned may result in ungaranteed behaviors)
-	return (*str == 0 && memcmp(str, str+1, size-1) == 0);
-}
-
-int towlower2(unsigned int c, Translator *translator)
-{
-	// check for non-standard upper to lower case conversions
-	if (c == 'I' && translator->langopts.dotless_i)
-		return 0x131; // I -> ı
-
-	return ucd_tolower(c);
-}
-
 static int IsRomanU(unsigned int c)
 {
 	if ((c == 'I') || (c == 'V') || (c == 'X') || (c == 'L'))
@@ -180,11 +166,10 @@ static void UngetC(int c)
 	ungot_char = c;
 }
 
-const char *WordToString2(unsigned int word)
+const char *WordToString2(char buf[5], unsigned int word)
 {
 	// Convert a language mnemonic word into a string
 	int ix;
-	static char buf[5];
 	char *p;
 
 	p = buf;
@@ -200,20 +185,17 @@ static const char *LookupSpecial(Translator *tr, const char *string, char *text_
 {
 	unsigned int flags[2];
 	char phonemes[55];
-	char phonemes2[55];
 	char *string1 = (char *)string;
 
 	flags[0] = flags[1] = 0;
 	if (LookupDictList(tr, &string1, phonemes, flags, 0, NULL)) {
-		SetWordStress(tr, phonemes, flags, -1, 0);
-		DecodePhonemes(phonemes, phonemes2);
-		sprintf(text_out, "[\002%s]]", phonemes2);
+		DecodeWithPhonemeMode(text_out, phonemes, tr, NULL, flags);
 		return text_out;
 	}
 	return NULL;
 }
 
-static const char *LookupCharName(Translator *tr, int c, int only)
+static const char *LookupCharName(char buf[60], Translator *tr, int c, bool only)
 {
 	// Find the phoneme string (in ascii) to speak the name of character c
 	// Used for punctuation characters and symbols
@@ -222,10 +204,8 @@ static const char *LookupCharName(Translator *tr, int c, int only)
 	unsigned int flags[2];
 	char single_letter[24];
 	char phonemes[60];
-	char phonemes2[60];
 	const char *lang_name = NULL;
 	char *string;
-	static char buf[60];
 
 	buf[0] = 0;
 	flags[0] = 0;
@@ -235,10 +215,12 @@ static const char *LookupCharName(Translator *tr, int c, int only)
 	ix = utf8_out(c, &single_letter[2]);
 	single_letter[2+ix] = 0;
 
-	if (only) {
+	if (only == true) {
 		string = &single_letter[2];
 		LookupDictList(tr, &string, phonemes, flags, 0, NULL);
-	} else {
+	}
+
+	if (only == false) {
 		string = &single_letter[1];
 		if (LookupDictList(tr, &string, phonemes, flags, 0, NULL) == 0) {
 			// try _* then *
@@ -249,61 +231,43 @@ static const char *LookupCharName(Translator *tr, int c, int only)
 				TranslateRules(tr, &single_letter[2], phonemes, sizeof(phonemes), NULL, 0, NULL);
 			}
 		}
-	}
-
-	if ((only == 0) && ((phonemes[0] == 0) || (phonemes[0] == phonSWITCH)) && (tr->translator_name != L('e', 'n'))) {
-		// not found, try English
-		SetTranslator2(ESPEAKNG_DEFAULT_VOICE);
-		string = &single_letter[1];
-		single_letter[1] = '_';
-		if (LookupDictList(translator2, &string, phonemes, flags, 0, NULL) == 0) {
-			string = &single_letter[2];
-			LookupDictList(translator2, &string, phonemes, flags, 0, NULL);
-		}
-		if (phonemes[0])
-			lang_name = ESPEAKNG_DEFAULT_VOICE;
-		else
-			SelectPhonemeTable(voice->phoneme_tab_ix); // revert to original phoneme table
+		
+		if (((phonemes[0] == 0) || (phonemes[0] == phonSWITCH)) && (tr->translator_name != L('e', 'n'))) {
+    		// not found, try English
+    		SetTranslator2(ESPEAKNG_DEFAULT_VOICE);
+    		string = &single_letter[1];
+    		single_letter[1] = '_';
+    		if (LookupDictList(translator2, &string, phonemes, flags, 0, NULL) == 0) {
+    			string = &single_letter[2];
+    			LookupDictList(translator2, &string, phonemes, flags, 0, NULL);
+    		}
+    		if (phonemes[0])
+    			lang_name = ESPEAKNG_DEFAULT_VOICE;
+    		else
+    			SelectPhonemeTable(voice->phoneme_tab_ix); // revert to original phoneme table
+    	}
 	}
 
 	if (phonemes[0]) {
 		if (lang_name) {
-			SetWordStress(translator2, phonemes, flags, -1, 0);
-			DecodePhonemes(phonemes, phonemes2);
-			sprintf(buf, "[\002_^_%s %s _^_%s]]", ESPEAKNG_DEFAULT_VOICE, phonemes2, WordToString2(tr->translator_name));
+			DecodeWithPhonemeMode(buf, phonemes, tr, translator2, flags);
 			SelectPhonemeTable(voice->phoneme_tab_ix); // revert to original phoneme table
 		} else {
-			SetWordStress(tr, phonemes, flags, -1, 0);
-			DecodePhonemes(phonemes, phonemes2);
-			sprintf(buf, "[\002%s]] ", phonemes2);
+			DecodeWithPhonemeMode(buf, phonemes, tr, NULL, flags);
 		}
-	} else if (only == 0)
+	} else if (only == false)
 		strcpy(buf, "[\002(X1)(X1)(X1)]]");
 
 	return buf;
 }
 
-int Read4Bytes(FILE *f)
-{
-	// Read 4 bytes (least significant first) into a word
-	int ix;
-	unsigned char c;
-	int acc = 0;
-
-	for (ix = 0; ix < 4; ix++) {
-		c = fgetc(f) & 0xff;
-		acc += (c << (ix*8));
-	}
-	return acc;
-}
-
 static int AnnouncePunctuation(Translator *tr, int c1, int *c2_ptr, char *output, int *bufix, int end_clause)
 {
 	// announce punctuation names
 	// c1:  the punctuation character
 	// c2:  the following character
 
-	int punct_count;
+
 	const char *punctname = NULL;
 	int soundicon;
 	int attributes;
@@ -312,8 +276,8 @@ static int AnnouncePunctuation(Translator *tr, int c1, int *c2_ptr, char *output
 	int len;
 	int bufix1;
 	char buf[200];
-	char buf2[80];
 	char ph_buf[30];
+	char cn_buf[60];
 
 	c2 = *c2_ptr;
 	buf[0] = 0;
@@ -328,14 +292,15 @@ static int AnnouncePunctuation(Translator *tr, int c1, int *c2_ptr, char *output
 				punctname = ph_buf; // use word for 'period' instead of 'dot'
 		}
 		if (punctname == NULL)
-			punctname = LookupCharName(tr, c1, 0);
+			punctname = LookupCharName(cn_buf, tr, c1, false);
 
 		if (punctname == NULL)
 			return -1;
 
+
 		if ((*bufix == 0) || (end_clause == 0) || (tr->langopts.param[LOPT_ANNOUNCE_PUNCT] & 2)) {
-			punct_count = 1;
-			while ((c2 == c1) && (c1 != '<')) { // don't eat extra '<', it can miss XML tags
+			int punct_count = 1;
+			while (!Eof() && (c2 == c1) && (c1 != '<')) { // don't eat extra '<', it can miss XML tags
 				punct_count++;
 				c2 = GetC();
 			}
@@ -350,6 +315,7 @@ static int AnnouncePunctuation(Translator *tr, int c1, int *c2_ptr, char *output
 				if (embedded_value[EMBED_S] < 300)
 					sprintf(buf, "\001+10S"); // Speak punctuation name faster, unless we are already speaking fast.  It would upset Sonic SpeedUp
 
+				char buf2[80];
 				while (punct_count-- > 0) {
 					sprintf(buf2, " %s", punctname);
 					strcat(buf, buf2);
@@ -364,10 +330,8 @@ static int AnnouncePunctuation(Translator *tr, int c1, int *c2_ptr, char *output
 				        punctname, punct_count, punctname);
 		} else {
 			// end the clause now and pick up the punctuation next time
-			UngetC(c2);
 			ungot_char2 = c1;
-			buf[0] = ' ';
-			buf[1] = 0;
+			TerminateBufWithSpaceAndZero(buf, 0, &c2);
 		}
 	}
 
@@ -407,7 +371,6 @@ int AddNameData(const char *name, int wide)
 
 	int ix;
 	int len;
-	void *vp;
 
 	if (wide) {
 		len = (wcslen((const wchar_t *)name)+1)*sizeof(wchar_t);
@@ -417,6 +380,7 @@ int AddNameData(const char *name, int wide)
 
 	if (namedata_ix+len >= n_namedata) {
 		// allocate more space for marker names
+		void *vp;
 		if ((vp = realloc(namedata, namedata_ix+len + 1000)) == NULL)
 			return -1;  // failed to allocate, original data is unchanged but ignore this new name
 		// !!! Bug?? If the allocated data shifts position, then pointers given to user application will be invalid
@@ -449,7 +413,7 @@ void SetVoiceStack(espeak_VOICE *v, const char *variant_name)
 	if (variant_name[0] == '!' && variant_name[1] == 'v' && variant_name[2] == PATHSEP)
 		variant_name += 3; // strip variant directory name, !v plus PATHSEP
 	strncpy0(base_voice_variant_name, variant_name, sizeof(base_voice_variant_name));
-	memcpy(&base_voice, &current_voice_selected, sizeof(base_voice));
+	memcpy(&base_voice, espeak_GetCurrentVoice(), sizeof(base_voice));
 }
 
 static void RemoveChar(char *p)
@@ -460,6 +424,45 @@ static void RemoveChar(char *p)
 	memset(p, ' ', utf8_in(&c, p));
 }
 
+static int lookupwchar2(const unsigned short *list, int c)
+{
+	// Replace character c by another character.
+	// Returns 0 = not found, 1 = delete character
+
+	int ix;
+
+	for (ix = 0; list[ix] != 0; ix += 2) {
+		if (list[ix] == c)
+			return list[ix+1];
+	}
+	return 0;
+}
+
+static bool IgnoreOrReplaceChar(Translator *tr, int *c1) {
+    int i;
+    if ((i = lookupwchar2(tr->chars_ignore, *c1)) != 0) {
+        if (i == 1) {
+            // ignore this character
+            return true;
+        }
+        *c1 = i; // replace current character with the result
+    }
+    return false;
+}
+
+static int CheckPhonemeMode(int option_phoneme_input, int phoneme_mode, int c1, int c2) {
+		if (option_phoneme_input) {
+			if (phoneme_mode > 0)
+				phoneme_mode--;
+			else if ((c1 == '[') && (c2 == '['))
+				phoneme_mode = -1; // input is phoneme mnemonics, so don't look for punctuation
+			else if ((c1 == ']') && (c2 == ']'))
+				phoneme_mode = 2; // set phoneme_mode to zero after the next two characters
+
+		}
+    return phoneme_mode;
+}
+
 int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_buf, int *tone_type, char *voice_change)
 {
 	/* Find the end of the current clause.
@@ -477,8 +480,7 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 	int c1 = ' '; // current character
 	int c2; // next character
 	int cprev = ' '; // previous character
-	int cprev2 = ' ';
-	int c_next;
+	int c_next = 0;
 	int parag;
 	int ix = 0;
 	int j;
@@ -535,14 +537,15 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 				return CLAUSE_NONE;
 			}
 		}
-
-		cprev2 = cprev;
+		int cprev2 = cprev;
 		cprev = c1;
 		c1 = c2;
 
 		if (ungot_string_ix >= 0) {
-			if (ungot_string[ungot_string_ix] == 0)
+			if (ungot_string[ungot_string_ix] == 0) {
+				MAKE_MEM_UNDEFINED(&ungot_string, sizeof(ungot_string));
 				ungot_string_ix = -1;
+			}
 		}
 
 		if ((ungot_string_ix == 0) && (ungot_char2 == 0))
@@ -591,10 +594,8 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 					// check for space in the output buffer for embedded commands produced by the SSML tag
 					if (ix > (n_buf - 20)) {
 						// Perhaps not enough room, end the clause before the SSML tag
-						UngetC(c2);
 						ungot_char2 = c1;
-						buf[ix] = ' ';
-						buf[ix+1] = 0;
+						TerminateBufWithSpaceAndZero(buf, ix, &c2);
 						return CLAUSE_NONE;
 					}
 
@@ -608,11 +609,12 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 					xml_buf[n_xml_buf] = 0;
 					c2 = ' ';
 
+					if (base_voice.identifier)
+						strcpy(current_voice_id, base_voice.identifier);
 					terminator = ProcessSsmlTag(xml_buf, buf, &ix, n_buf, xmlbase, &audio_text, current_voice_id, &base_voice, base_voice_variant_name, &ignore_text, &clear_skipping_text, &sayas_mode, &sayas_start, ssml_stack, &n_ssml_stack, &n_param_stack, (int *)speech_parameters);
 
 					if (terminator != 0) {
-						buf[ix] = ' ';
-						buf[ix++] = 0;
+						TerminateBufWithSpaceAndZero(buf, ix, NULL);
 
 						if (terminator & CLAUSE_TYPE_VOICE_CHANGE)
 							strcpy(voice_change, current_voice_id);
@@ -638,8 +640,7 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 				ix += utf8_out(c1, &buf[ix]);
 				terminator = CLAUSE_PERIOD; // line doesn't end in punctuation, assume period
 			}
-			buf[ix] = ' ';
-			buf[ix+1] = 0;
+			TerminateBufWithSpaceAndZero(buf, ix, NULL);
 			return terminator;
 		}
 
@@ -647,7 +648,7 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
  			// an embedded command. If it's a voice change, end the clause
 			if (c2 == 'V') {
 				buf[ix++] = 0; // end the clause at this point
-				while (!iswspace(c1 = GetC()) && !Eof() && (ix < (n_buf-1)))
+				while (!Eof() && !iswspace(c1 = GetC()) && (ix < (n_buf-1)))
 					buf[ix++] = c1; // add voice name to end of buffer, after the text
 				buf[ix++] = 0;
 				return CLAUSE_VOICE;
@@ -657,7 +658,7 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 				strcpy(&buf[ix], "   ");
 				ix += 3;
 
-				if ((c2 = GetC()) == '0')
+				if (!Eof() && (c2 = GetC()) == '0')
 					option_punctuation = 0;
 				else {
 					option_punctuation = 1;
@@ -665,7 +666,7 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 					if (c2 != '1') {
 						// a list of punctuation characters to be spoken, terminated by space
 						j = 0;
-						while (!iswspace(c2) && !Eof()) {
+						while (!Eof() && !iswspace(c2) && (j < N_PUNCTLIST-1)) {
 							option_punctlist[j++] = c2;
 							c2 = GetC();
 							buf[ix++] = ' ';
@@ -674,20 +675,17 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 						option_punctuation = 2;
 					}
 				}
-				c2 = GetC();
+				if (!Eof())
+					c2 = GetC();
 				continue;
 			}
 		}
 
 		linelength++;
 
-		if ((j = lookupwchar2(tr->chars_ignore, c1)) != 0) {
-			if (j == 1) {
-				// ignore this character (eg. zero-width-non-joiner U+200C)
-				continue;
-			}
-			c1 = j; // replace the character
-		}
+		 if (IgnoreOrReplaceChar(tr, &c1) == true)
+		    continue;
+
 
 		if (iswalnum(c1))
 			any_alnum = true;
@@ -725,14 +723,7 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 		} else if (iswalpha(c1))
 			tr->clause_lower_count++;
 
-		if (option_phoneme_input) {
-			if (phoneme_mode > 0)
-				phoneme_mode--;
-			else if ((c1 == '[') && (c2 == '['))
-				phoneme_mode = -1; // input is phoneme mnemonics, so don't look for punctuation
-			else if ((c1 == ']') && (c2 == ']'))
-				phoneme_mode = 2; // set phoneme_mode to zero after the next two characters
-		}
+		phoneme_mode = CheckPhonemeMode(option_phoneme_input, phoneme_mode, c1, c2);
 
 		if (c1 == '\n') {
 			parag = 0;
@@ -745,12 +736,9 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 			}
 			if (parag > 0) {
 				// 2nd newline, assume paragraph
-				UngetC(c2);
-
 				if (end_clause_after_tag)
 					RemoveChar(&buf[end_clause_index]); // delete clause-end punctiation
-				buf[ix] = ' ';
-				buf[ix+1] = 0;
+				TerminateBufWithSpaceAndZero(buf, ix, &c2);
 				if (parag > 3)
 					parag = 3;
 				if (option_ssml) parag = 1;
@@ -759,9 +747,7 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 
 			if (linelength <= option_linelength) {
 				// treat lines shorter than a specified length as end-of-clause
-				UngetC(c2);
-				buf[ix] = ' ';
-				buf[ix+1] = 0;
+				TerminateBufWithSpaceAndZero(buf, ix, &c2);
 				return CLAUSE_COLON;
 			}
 
@@ -780,10 +766,8 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 
 				if (!iswspace(c1)) {
 					if (!IsAlpha(c1) || !iswlower(c1)) {
-						UngetC(c2);
 						ungot_char2 = c1;
-						buf[end_clause_index] = ' '; // delete the end-clause punctuation
-						buf[end_clause_index+1] = 0;
+						TerminateBufWithSpaceAndZero(buf, end_clause_index, &c2);
 						return end_clause_after_tag;
 					}
 					end_clause_after_tag = 0;
@@ -791,7 +775,7 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 			}
 
 			if ((c1 == '.') && (c2 == '.')) {
-				while ((c_next = GetC()) == '.') {
+				while (!Eof() && (c_next = GetC()) == '.') {
 					// 3 or more dots, replace by elipsis
 					c1 = 0x2026;
 					c2 = ' ';
@@ -808,7 +792,7 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 				// Handling of sequences of ? and ! like ??!?, !!??!, ?!! etc
 				// Use only first char as determinant
 				if(punct_data & (CLAUSE_QUESTION | CLAUSE_EXCLAMATION)) {
-					while(clause_type_from_codepoint(c2) & (CLAUSE_QUESTION | CLAUSE_EXCLAMATION)) {
+					while(!Eof() && clause_type_from_codepoint(c2) & (CLAUSE_QUESTION | CLAUSE_EXCLAMATION)) {
 						c_next = GetC();
 						c2 = c_next;
 					}
@@ -845,7 +829,8 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 				char *p2;
 
 				p2 = &buf[ix];
-				sprintf(p2, "%s", LookupCharName(tr, c1, 1));
+				char cn_buf[60];
+				sprintf(p2, "%s", LookupCharName(cn_buf, tr, c1, true));
 				if (p2[0] != 0) {
 					ix += strlen(p2);
 					announced_punctuation = c1;
@@ -916,9 +901,7 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 				}
 
 				if (is_end_clause) {
-					UngetC(c_next);
-					buf[ix] = ' ';
-					buf[ix+1] = 0;
+					TerminateBufWithSpaceAndZero(buf, ix, &c_next);
 
 					if (iswdigit(cprev) && !IsAlpha(c_next)) // ????
 						punct_data &= ~CLAUSE_DOT_AFTER_LAST_WORD;
@@ -963,9 +946,7 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 			// clause too long, getting near end of buffer, so break here
 			// try to break at a word boundary (unless we actually reach the end of buffer).
 			// (n_buf-4) is to allow for 3 bytes of multibyte character plus terminator.
-			buf[ix] = ' ';
-			buf[ix+1] = 0;
-			UngetC(c2);
+			TerminateBufWithSpaceAndZero(buf, ix, &c2);
 			return CLAUSE_NONE;
 		}
 	}
@@ -974,8 +955,7 @@ int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_
 		ix += utf8_out(CHAR_EMPHASIS, &buf[ix]);
 	if (end_clause_after_tag)
 		RemoveChar(&buf[end_clause_index]); // delete clause-end punctiation
-	buf[ix] = ' ';
-	buf[ix+1] = 0;
+	TerminateBufWithSpaceAndZero(buf, ix, NULL);
 	return CLAUSE_EOF; // end of file
 }
 
@@ -997,7 +977,9 @@ void InitText2(void)
 	ungot_char2 = 0;
 
 	n_ssml_stack = 1;
+	MAKE_MEM_UNDEFINED(&ssml_stack[1], sizeof(ssml_stack) - sizeof(ssml_stack[0]));
 	n_param_stack = 1;
+	MAKE_MEM_UNDEFINED(&param_stack[1], sizeof(param_stack) - sizeof(param_stack[0]));
 	ssml_stack[0].tag_type = 0;
 
 	for (param = 0; param < N_SPEECH_PARAM; param++)
@@ -1016,3 +998,26 @@ void InitText2(void)
 
 	xmlbase = NULL;
 }
+
+static void TerminateBufWithSpaceAndZero(char *buf, int index, int *ungetc) {
+	buf[index] = ' ';
+	buf[index+1] = 0;
+
+	if (ungetc != NULL) {
+		UngetC(*ungetc);
+	}
+}
+
+static void DecodeWithPhonemeMode(char *buf, char *phonemes, Translator *tr, Translator *tr2, unsigned int flags[]) {
+	char phonemes2[55];
+	if (tr2 == NULL) {
+		SetWordStress(tr, phonemes, flags, -1, 0);
+		DecodePhonemes(phonemes, phonemes2);
+		sprintf(buf, "[\002%s]]", phonemes2);
+	} else {
+		SetWordStress(tr2, phonemes, flags, -1, 0);
+	    DecodePhonemes(phonemes, phonemes2);
+			char wbuf[5];
+	    sprintf(buf, "[\002_^_%s %s _^_%s]]", ESPEAKNG_DEFAULT_VOICE, phonemes2, WordToString2(wbuf, tr->translator_name));
+    }
+}
