@@ -40,15 +40,16 @@
 #include <ucd/ucd.h>
 
 #include "ssml.h"
-#include "dictionary.h"           // for strncpy0
+#include "common.h"           // for strncpy0
 #include "mnemonics.h"               // for LookupMnemName, MNEM_TAB, 
 #include "readclause.h"           // for PARAM_STACK, param_stack, AddNameData
 #include "soundicon.h"               // for LoadSoundFile2
 #include "synthesize.h"           // for SPEED_FACTORS, speed
-#include "translate.h"            // for CTRL_EMBEDDED, IsDigit09, utf8_out
+#include "translate.h"            // for CTRL_EMBEDDED
 #include "voice.h"                // for SelectVoice, SelectVoiceByName
+#include "speech.h"               // for MAKE_MEM_UNDEFINED
 
-static MNEM_TAB ssmltags[] = {
+static const MNEM_TAB ssmltags[] = {
 	{ "speak",     SSML_SPEAK },
 	{ "voice",     SSML_VOICE },
 	{ "prosody",   SSML_PROSODY },
@@ -86,6 +87,8 @@ static MNEM_TAB ssmltags[] = {
 	{ NULL, 0 }
 };
 
+static int (*uri_callback)(int, const char *, const char *) = NULL;
+
 static int attrcmp(const wchar_t *string1, const char *string2)
 {
 	int ix;
@@ -131,16 +134,21 @@ static int attrnumber(const wchar_t *pw, int default_value, int type)
 static int attrcopy_utf8(char *buf, const wchar_t *pw, int len)
 {
 	// Convert attribute string into utf8, write to buf, and return its utf8 length
-	unsigned int c;
 	int ix = 0;
-	int n;
-	int prev_c = 0;
 
 	if (pw != NULL) {
+		wchar_t quote = pw[-1];
+		if ((quote != '"') && (quote != '\'')) quote = 0;
+
+		unsigned int c;
+		int prev_c = 0;
 		while ((ix < (len-4)) && ((c = *pw++) != 0)) {
-			if ((c == '"') && (prev_c != '\\'))
+			if ((quote == 0) && (isspace(c) || (c == '/')))
+				break;
+			if ((quote != 0) && (c == quote) && (prev_c != '\\'))
 				break; // " indicates end of attribute, unless preceded by backstroke
-			n = utf8_out(c, &buf[ix]);
+
+			int n = utf8_out(c, &buf[ix]);
 			ix += n;
 			prev_c = c;
 		}
@@ -207,12 +215,13 @@ static const char *VoiceFromStack(SSML_STACK *ssml_stack, int n_ssml_stack, espe
 	const char *p;
 	SSML_STACK *sp;
 	const char *v_id;
-	int voice_name_specified;
 	int voice_found;
 	espeak_VOICE voice_select;
 	static char voice_name[40];
+	static char identifier[40];
 	char language[40];
-	char buf[80];
+
+	MAKE_MEM_UNDEFINED(&voice_name, sizeof(voice_name));
 
 	strcpy(voice_name, ssml_stack[0].voice_name);
 	strcpy(language, ssml_stack[0].language);
@@ -222,12 +231,14 @@ static const char *VoiceFromStack(SSML_STACK *ssml_stack, int n_ssml_stack, espe
 	voice_select.identifier = NULL;
 
 	for (ix = 0; ix < n_ssml_stack; ix++) {
+		espeak_VOICE *v;
 		sp = &ssml_stack[ix];
-		voice_name_specified = 0;
+		int voice_name_specified = 0;
 
-		if ((sp->voice_name[0] != 0) && (SelectVoiceByName(NULL, sp->voice_name) != NULL)) {
+		if ((sp->voice_name[0] != 0) && ((v = SelectVoiceByName(NULL, sp->voice_name)) != NULL)) {
 			voice_name_specified = 1;
 			strcpy(voice_name, sp->voice_name);
+			strcpy(identifier, v->identifier);
 			language[0] = 0;
 			voice_select.gender = ENGENDER_UNKNOWN;
 			voice_select.age = 0;
@@ -248,7 +259,10 @@ static const char *VoiceFromStack(SSML_STACK *ssml_stack, int n_ssml_stack, espe
 			}
 
 			if (voice_name_specified == 0)
+			{
 				voice_name[0] = 0; // forget a previous voice name if a language is specified
+				identifier[0] = 0;
+			}
 		}
 		if (sp->voice_gender != ENGENDER_UNKNOWN)
 			voice_select.gender = sp->voice_gender;
@@ -260,13 +274,16 @@ static const char *VoiceFromStack(SSML_STACK *ssml_stack, int n_ssml_stack, espe
 	}
 
 	voice_select.name = voice_name;
+	voice_select.identifier = identifier;
 	voice_select.languages = language;
+
 	v_id = SelectVoice(&voice_select, &voice_found);
 	if (v_id == NULL)
 		return "default";
 
 	if ((strchr(v_id, '+') == NULL) && ((voice_select.gender == ENGENDER_UNKNOWN) || (voice_select.gender == base_voice->gender)) && (base_voice_variant_name[0] != 0)) {
 		// a voice variant has not been selected, use the original voice variant
+		char buf[80];
 		sprintf(buf, "%s+%s", v_id, base_voice_variant_name);
 		strncpy0(voice_name, buf, sizeof(voice_name));
 		return voice_name;
@@ -275,13 +292,13 @@ static const char *VoiceFromStack(SSML_STACK *ssml_stack, int n_ssml_stack, espe
 }
 
 
-static wchar_t *GetSsmlAttribute(wchar_t *pw, const char *name)
+static const wchar_t *GetSsmlAttribute(wchar_t *pw, const char *name)
 {
 	// Gets the value string for an attribute.
 	// Returns NULL if the attribute is not present
 
 	int ix;
-	static wchar_t empty[1] = { 0 };
+	static const wchar_t empty[1] = { 0 };
 
 	while (*pw != 0) {
 		if (iswspace(pw[-1])) {
@@ -297,8 +314,10 @@ static wchar_t *GetSsmlAttribute(wchar_t *pw, const char *name)
 				while (iswspace(*pw)) pw++;
 				if ((*pw == '"') || (*pw == '\'')) // allow single-quotes ?
 					return pw+1;
-				else
+				else if (iswspace(*pw) || (*pw == '/')) // end of attribute
 					return empty;
+				else
+					return pw;
 			}
 		}
 		pw++;
@@ -315,12 +334,6 @@ static int GetVoiceAttributes(wchar_t *pw, int tag_type, SSML_STACK *ssml_sp, SS
 	// a voice change.
 	// Returns  CLAUSE_TYPE_VOICE_CHANGE if there is a voice change
 
-	wchar_t *lang;
-	wchar_t *gender;
-	wchar_t *name;
-	wchar_t *age;
-	wchar_t *variant;
-	int value;
 	const char *new_voice_id;
 
 	static const MNEM_TAB mnem_gender[] = {
@@ -335,6 +348,12 @@ static int GetVoiceAttributes(wchar_t *pw, int tag_type, SSML_STACK *ssml_sp, SS
 		if (n_ssml_stack > 1)
 			n_ssml_stack--;
 	} else {
+		const wchar_t *lang;
+    	const wchar_t *gender;
+    	const wchar_t *name;
+    	const wchar_t *age;
+    	const wchar_t *variant;
+
 		// add a stack frame if any voice details are specified
 		lang = GetSsmlAttribute(pw, "xml:lang");
 
@@ -356,6 +375,8 @@ static int GetVoiceAttributes(wchar_t *pw, int tag_type, SSML_STACK *ssml_sp, SS
 
 		ssml_sp = &ssml_stack[n_ssml_stack++];
 
+		int value;
+
 		attrcopy_utf8(ssml_sp->language, lang, sizeof(ssml_sp->language));
 		attrcopy_utf8(ssml_sp->voice_name, name, sizeof(ssml_sp->voice_name));
 		if ((value = attrnumber(variant, 1, 0)) > 0)
@@ -381,10 +402,9 @@ static void ProcessParamStack(char *outbuf, int *outix, int n_param_stack, PARAM
 	// Set the speech parameters from the parameter stack
 	int param;
 	int ix;
-	int value;
 	char buf[20];
 	int new_parameters[N_SPEECH_PARAM];
-	static char cmd_letter[N_SPEECH_PARAM] = { 0, 'S', 'A', 'P', 'R', 0, 'C', 0, 0, 0, 0, 0, 'F' }; // embedded command letters
+	static const char cmd_letter[N_SPEECH_PARAM] = { 0, 'S', 'A', 'P', 'R', 0, 'C', 0, 0, 0, 0, 0, 'F' }; // embedded command letters
 
 	for (param = 0; param < N_SPEECH_PARAM; param++)
 		new_parameters[param] = -1;
@@ -397,6 +417,7 @@ static void ProcessParamStack(char *outbuf, int *outix, int n_param_stack, PARAM
 	}
 
 	for (param = 0; param < N_SPEECH_PARAM; param++) {
+		int value;
 		if ((value = new_parameters[param]) != speech_parameters[param]) {
 			buf[0] = 0;
 
@@ -460,7 +481,7 @@ static void PopParamStack(int tag_type, char *outbuf, int *outix, int *n_param_s
 static int ReplaceKeyName(char *outbuf, int index, int *outix)
 {
 	// Replace some key-names by single characters, so they can be pronounced in different languages
-	static MNEM_TAB keynames[] = {
+	static const MNEM_TAB keynames[] = {
 		{ "space ",        0xe020 },
 		{ "tab ",          0xe009 },
 		{ "underscore ",   0xe05f },
@@ -468,24 +489,24 @@ static int ReplaceKeyName(char *outbuf, int index, int *outix)
 		{ NULL,            0 }
 	};
 
-	int ix;
 	int letter;
 	char *p;
 
 	p = &outbuf[index];
 
 	if ((letter = LookupMnem(keynames, p)) != 0) {
-		ix = utf8_out(letter, p);
+		int ix;
+		 ix = utf8_out(letter, p);
 		*outix = index + ix;
 		return letter;
 	}
 	return 0;
 }
 
-static void SetProsodyParameter(int param_type, wchar_t *attr1, PARAM_STACK *sp, PARAM_STACK *param_stack, int *speech_parameters)
+static void SetProsodyParameter(int param_type, const wchar_t *attr1, PARAM_STACK *sp, PARAM_STACK *param_stack, int *speech_parameters)
 {
 	int value;
-	int sign;
+
 
 	static const MNEM_TAB mnem_volume[] = {
 		{ "default", 100 },
@@ -528,7 +549,7 @@ static void SetProsodyParameter(int param_type, wchar_t *attr1, PARAM_STACK *sp,
 		{ NULL,       -1 }
 	};
 
-	static const MNEM_TAB *mnem_tabs[5] = {
+	static const MNEM_TAB * const mnem_tabs[5] = {
 		NULL, mnem_rate, mnem_volume, mnem_pitch, mnem_range
 	};
 
@@ -536,7 +557,7 @@ static void SetProsodyParameter(int param_type, wchar_t *attr1, PARAM_STACK *sp,
 		// mnemonic specifies a value as a percentage of the base pitch/range/rate/volume
 		sp->parameter[param_type] = (param_stack[0].parameter[param_type] * value)/100;
 	} else {
-		sign = attr_prosody_value(param_type, attr1, &value);
+		int sign = attr_prosody_value(param_type, attr1, &value);
 
 		if (sign == 0)
 			sp->parameter[param_type] = value; // absolute value in Hz
@@ -557,21 +578,19 @@ int ProcessSsmlTag(wchar_t *xml_buf, char *outbuf, int *outix, int n_outbuf, con
 
 	unsigned int ix;
 	int index;
-	int c;
 	int tag_type;
 	int value;
 	int value2;
 	int value3;
 	int voice_change_flag;
 	wchar_t *px;
-	wchar_t *attr1;
-	wchar_t *attr2;
-	wchar_t *attr3;
+	const wchar_t *attr1;
+	const wchar_t *attr2;
+	const wchar_t *attr3;
 	int terminator;
-	char *uri;
 	int param_type;
 	char tag_name[40];
-	char buf[80];
+	char buf[160];
 	PARAM_STACK *sp;
 	SSML_STACK *ssml_sp;
 
@@ -581,7 +600,7 @@ int ProcessSsmlTag(wchar_t *xml_buf, char *outbuf, int *outix, int n_outbuf, con
 		}
 
 	// these tags have no effect if they are self-closing, eg. <voice />
-	static char ignore_if_self_closing[] = { 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0, 0, 0, 0 };
+	static const char ignore_if_self_closing[] = { 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0, 0, 0, 0 };
 
 	bool self_closing = false;
 	int len;
@@ -645,11 +664,12 @@ int ProcessSsmlTag(wchar_t *xml_buf, char *outbuf, int *outix, int n_outbuf, con
 		{ NULL,      -1 }
 	};
 
-	static const char *prosody_attr[5] = {
+	static const char * const prosody_attr[5] = {
 		NULL, "rate", "volume", "pitch", "range"
 	};
 
 	for (ix = 0; ix < (sizeof(tag_name)-1); ix++) {
+		int c;
 		if (((c = xml_buf[ix]) == 0) || iswspace(c))
 			break;
 		tag_name[ix] = tolower((char)c);
@@ -711,13 +731,13 @@ int ProcessSsmlTag(wchar_t *xml_buf, char *outbuf, int *outix, int n_outbuf, con
 			value = attrlookup(attr1, mnem_emphasis);
 
 		if (translator->langopts.tone_language == 1) {
-			static unsigned char emphasis_to_pitch_range[] = { 50, 50, 40, 70, 90, 100 };
-			static unsigned char emphasis_to_volume[] = { 100, 100, 70, 110, 135, 150 };
+			static const unsigned char emphasis_to_pitch_range[] = { 50, 50, 40, 70, 90, 100 };
+			static const unsigned char emphasis_to_volume[] = { 100, 100, 70, 110, 135, 150 };
 			// tone language (eg.Chinese) do emphasis by increasing the pitch range.
 			sp->parameter[espeakRANGE] = emphasis_to_pitch_range[value];
 			sp->parameter[espeakVOLUME] = emphasis_to_volume[value];
 		} else {
-			static unsigned char emphasis_to_volume2[] = { 100, 100, 75, 100, 120, 150 };
+			static const unsigned char emphasis_to_volume2[] = { 100, 100, 75, 100, 120, 150 };
 			sp->parameter[espeakVOLUME] = emphasis_to_volume2[value];
 			sp->parameter[espeakEMPHASIS] = value;
 		}
@@ -794,7 +814,7 @@ int ProcessSsmlTag(wchar_t *xml_buf, char *outbuf, int *outix, int n_outbuf, con
 			// add name to circular buffer of marker names
 			attrcopy_utf8(buf, attr1, sizeof(buf));
 
-			if (strcmp(skip_marker, buf) == 0) {
+			if ((buf[0] != 0) && (strcmp(skip_marker, buf) == 0)) {
 				// This is the marker we are waiting for before starting to speak
 				*clear_skipping_text = true;
 				skip_marker[0] = 0;
@@ -812,11 +832,11 @@ int ProcessSsmlTag(wchar_t *xml_buf, char *outbuf, int *outix, int n_outbuf, con
 		sp = PushParamStack(tag_type, n_param_stack, (PARAM_STACK *)param_stack);
 
 		if ((attr1 = GetSsmlAttribute(px, "src")) != NULL) {
-			char fname[256];
 			attrcopy_utf8(buf, attr1, sizeof(buf));
 
 			if (uri_callback == NULL) {
 				if ((xmlbase != NULL) && (buf[0] != '/')) {
+					char fname[256];
 					sprintf(fname, "%s/%s", xmlbase, buf);
 					index = LoadSoundFile2(fname);
 				} else
@@ -829,6 +849,7 @@ int ProcessSsmlTag(wchar_t *xml_buf, char *outbuf, int *outix, int n_outbuf, con
 				}
 			} else {
 				if ((index = AddNameData(buf, 0)) >= 0) {
+					char *uri;
 					uri = &namedata[index];
 					if (uri_callback(1, uri, xmlbase) == 0) {
 						sprintf(buf, "%c%dU", CTRL_EMBEDDED, index);
@@ -855,8 +876,9 @@ int ProcessSsmlTag(wchar_t *xml_buf, char *outbuf, int *outix, int n_outbuf, con
 		terminator = CLAUSE_NONE;
 
 		if ((attr1 = GetSsmlAttribute(px, "strength")) != NULL) {
-			static int break_value[6] = { 0, 7, 14, 21, 40, 80 }; // *10mS
+			static const int break_value[6] = { 0, 7, 14, 21, 40, 80 }; // *10mS
 			value = attrlookup(attr1, mnem_break);
+			if (value < 0) value = 2;
 			if (value < 3) {
 				// adjust prepause on the following word
 				sprintf(&outbuf[*outix], "%c%dB", CTRL_EMBEDDED, value);
@@ -868,6 +890,19 @@ int ProcessSsmlTag(wchar_t *xml_buf, char *outbuf, int *outix, int n_outbuf, con
 		if ((attr2 = GetSsmlAttribute(px, "time")) != NULL) {
 			value2 = attrnumber(attr2, 0, 1);   // pause in mS
 
+			value2 = value2 * speech_parameters[espeakSSML_BREAK_MUL] / 100;
+
+			int wpm = speech_parameters[espeakRATE];
+			espeak_SetParameter(espeakRATE, wpm, 0);
+
+			#if USE_LIBSONIC
+			if (wpm >= espeakRATE_MAXIMUM) {
+				// Compensate speedup with libsonic, see function SetSpeed()
+				double sonic = ((double)wpm)/espeakRATE_NORMAL;
+				value2 = value2 * sonic;
+			}
+			#endif
+
 			// compensate for speaking speed to keep constant pause length, see function PauseLength()
 			// 'value' here is x 10mS
 			value = (value2 * 256) / (speed.clause_pause_factor * 10);
@@ -951,7 +986,14 @@ int ProcessSsmlTag(wchar_t *xml_buf, char *outbuf, int *outix, int n_outbuf, con
 	return 0;
 }
 
-static MNEM_TAB xml_entity_mnemonics[] = {
+#pragma GCC visibility push(default)
+ESPEAK_API void espeak_SetUriCallback(int (*UriCallback)(int, const char *, const char *))
+{
+	uri_callback = UriCallback;
+}
+#pragma GCC visibility pop
+
+static const MNEM_TAB xml_entity_mnemonics[] = {
 	{ "gt",   '>' },
 	{ "lt",   0xe000 + '<' },   // private usage area, to avoid confusion with XML tag
 	{ "amp",  '&' },
