@@ -43,6 +43,7 @@
 #include "translate.h"            // for translator, LANGUAGE_OPTIONS, Trans...
 #include "voice.h"                // for voice_t, voice, LoadVoiceVariant
 #include "wavegen.h"              // for WcmdqInc, WcmdqFree, WcmdqStop
+#include "speech.h"               // for MAKE_MEM_UNDEFINED
 
 static void SmoothSpect(void);
 
@@ -67,15 +68,16 @@ static int syllable_centre;
 
 static voice_t *new_voice = NULL;
 
+static int (*phoneme_callback)(const char *) = NULL;
+
 #define RMS_GLOTTAL1 35   // vowel before glottal stop
 #define RMS_START 28  // 28
 #define VOWEL_FRONT_LENGTH  50
 
-const char *WordToString(unsigned int word)
+const char *WordToString(char buf[5], unsigned int word)
 {
 	// Convert a phoneme mnemonic word into a string
 	int ix;
-	static char buf[5];
 
 	for (ix = 0; ix < 4; ix++)
 		buf[ix] = word >> (ix*8);
@@ -83,7 +85,7 @@ const char *WordToString(unsigned int word)
 	return buf;
 }
 
-void SynthesizeInit()
+void SynthesizeInit(void)
 {
 	last_pitch_cmd = 0;
 	last_amp_cmd = 0;
@@ -118,7 +120,7 @@ static void EndPitch(int voice_break)
 	}
 }
 
-static void DoAmplitude(int amp, unsigned char *amp_env)
+static void DoAmplitude(int amp, const unsigned char *amp_env)
 {
 	intptr_t *q;
 
@@ -141,7 +143,7 @@ static void DoPhonemeAlignment(char* pho, int type)
 	WcmdqInc();
 }
 
-static void DoPitch(unsigned char *env, int pitch1, int pitch2)
+static void DoPitch(const unsigned char *env, int pitch1, int pitch2)
 {
 	intptr_t *q;
 
@@ -190,7 +192,6 @@ static void DoPause(int length, int control)
 	// control = 1, less shortening at fast speeds
 
 	unsigned int len;
-	int srate2;
 
 	if (length == 0)
 		len = 0;
@@ -200,7 +201,7 @@ static void DoPause(int length, int control)
 		if (len < 90000)
 			len = (len * samplerate) / 1000; // convert from mS to number of samples
 		else {
-			srate2 = samplerate / 25; // avoid overflow
+			int srate2 = samplerate / 25; // avoid overflow
 			len = (len * srate2) / 40;
 		}
 	}
@@ -375,7 +376,7 @@ int DoSample3(PHONEME_DATA *phdata, int length_mod, int amp)
 	return len;
 }
 
-static frame_t *AllocFrame()
+static frame_t *AllocFrame(void)
 {
 	// Allocate a temporary spectrum frame for the wavegen queue. Use a pool which is big
 	// enough to use a round-robin without checks.
@@ -388,6 +389,8 @@ static frame_t *AllocFrame()
 	ix++;
 	if (ix >= N_FRAME_POOL)
 		ix = 0;
+
+	MAKE_MEM_UNDEFINED(&frame_pool[ix], sizeof(frame_pool[ix]));
 	return &frame_pool[ix];
 }
 
@@ -397,7 +400,6 @@ static void set_frame_rms(frame_t *fr, int new_rms)
 	// RMS just adjust the formant amplitudes by the appropriate ratio
 
 	int x;
-	int h;
 	int ix;
 
 	static const short sqrt_tab[200] = {
@@ -429,6 +431,7 @@ static void set_frame_rms(frame_t *fr, int new_rms)
 	x = sqrt_tab[x]; // sqrt(new_rms/fr->rms)*0x200;
 
 	for (ix = 0; ix < 8; ix++) {
+		int h;
 		h = fr->fheight[ix] * x;
 		fr->fheight[ix] = h/0x200;
 	}
@@ -437,13 +440,11 @@ static void set_frame_rms(frame_t *fr, int new_rms)
 static void formants_reduce_hf(frame_t *fr, int level)
 {
 	// change height of peaks 2 to 8, percentage
-	int ix;
-	int x;
-
 	if (voice->klattv[0])
 		return;
 
-	for (ix = 2; ix < 8; ix++) {
+	for (int ix = 2; ix < 8; ix++) {
+		int x;
 		x = fr->fheight[ix] * level;
 		fr->fheight[ix] = x/100;
 	}
@@ -536,10 +537,6 @@ static int VowelCloseness(frame_t *fr)
 
 int FormantTransition2(frameref_t *seq, int *n_frames, unsigned int data1, unsigned int data2, PHONEME_TAB *other_ph, int which)
 {
-	int ix;
-	int formant;
-	int next_rms;
-
 	int len;
 	int rms;
 	int f1;
@@ -553,7 +550,7 @@ int FormantTransition2(frameref_t *seq, int *n_frames, unsigned int data1, unsig
 
 	#define N_VCOLOUR 2
 	// percentage change for each formant in 256ths
-	static short vcolouring[N_VCOLOUR][5] = {
+	static const short vcolouring[N_VCOLOUR][5] = {
 		{ 243, 272, 256, 256, 256 }, // palatal consonant follows
 		{ 256, 256, 240, 240, 240 }, // retroflex
 	};
@@ -588,7 +585,7 @@ int FormantTransition2(frameref_t *seq, int *n_frames, unsigned int data1, unsig
 		seq[0].frflags |= FRFLAG_LEN_MOD2; // reduce length modification
 		fr->frflags |= FRFLAG_LEN_MOD2;
 
-		next_rms = seq[1].frame->rms;
+		int next_rms = seq[1].frame->rms;
 
 		if (voice->klattv[0])
 			fr->klattp[KLATT_AV] = seq[1].frame->klattp[KLATT_AV] - 4;
@@ -632,11 +629,11 @@ int FormantTransition2(frameref_t *seq, int *n_frames, unsigned int data1, unsig
 			set_frame_rms(fr, rms);
 
 			if ((vcolour > 0) && (vcolour <= N_VCOLOUR)) {
-				for (ix = 0; ix < *n_frames; ix++) {
+				for (int ix = 0; ix < *n_frames; ix++) {
 					fr = CopyFrame(seq[ix].frame, 0);
 					seq[ix].frame = fr;
 
-					for (formant = 1; formant <= 5; formant++) {
+					for (int formant = 1; formant <= 5; formant++) {
 						int x;
 						x = fr->ffreq[formant] * vcolouring[vcolour-1][formant-1];
 						fr->ffreq[formant] = x / 256;
@@ -849,12 +846,8 @@ int DoSpect2(PHONEME_TAB *this_ph, int which, FMT_PARAMS *fmt_params,  PHONEME_L
 	int frameix;
 	frame_t *frame1;
 	frame_t *frame2;
-	frame_t *fr;
-	int ix;
 	intptr_t *q;
 	int len;
-	int frame_length;
-	int length_factor;
 	int length_mod;
 	int length_sum;
 	int length_min;
@@ -922,8 +915,9 @@ int DoSpect2(PHONEME_TAB *this_ph, int which, FMT_PARAMS *fmt_params,  PHONEME_L
 
 			if (last_frame->frflags & FRFLAG_BREAK_LF) {
 				// but flag indicates keep HF peaks in last segment
+				frame_t *fr;
 				fr = CopyFrame(frame1, 1);
-				for (ix = 3; ix < 8; ix++) {
+				for (int ix = 3; ix < 8; ix++) {
 					if (ix < 7)
 						fr->ffreq[ix] = last_frame->ffreq[ix];
 					fr->fheight[ix] = last_frame->fheight[ix];
@@ -942,13 +936,13 @@ int DoSpect2(PHONEME_TAB *this_ph, int which, FMT_PARAMS *fmt_params,  PHONEME_L
 
 	length_sum = 0;
 	for (frameix = 1; frameix < n_frames; frameix++) {
-		length_factor = length_mod;
+		int length_factor = length_mod;
 		if (frames[frameix-1].frflags & FRFLAG_LEN_MOD) // reduce effect of length mod
 			length_factor = (length_mod*(256-speed.lenmod_factor) + 256*speed.lenmod_factor)/256;
 		else if (frames[frameix-1].frflags & FRFLAG_LEN_MOD2) // reduce effect of length mod, used for the start of a vowel
 			length_factor = (length_mod*(256-speed.lenmod2_factor) + 256*speed.lenmod2_factor)/256;
 
-		frame_length = frames[frameix-1].length;
+		int frame_length = frames[frameix-1].length;
 		len = (frame_length * samplerate)/1000;
 		len = (len * length_factor)/256;
 		length_sum += len;
@@ -967,7 +961,7 @@ int DoSpect2(PHONEME_TAB *this_ph, int which, FMT_PARAMS *fmt_params,  PHONEME_L
 		if ((fmt_params->wav_addr != 0) && ((frame1->frflags & FRFLAG_DEFER_WAV) == 0)) {
 			// there is a wave file to play along with this synthesis
 			seq_len_adjust = 0;
-	
+
 			int wavefile_amp;
 			if (fmt_params->wav_amp == 0)
 				wavefile_amp = 32;
@@ -1038,19 +1032,15 @@ void DoPhonemeMarker(int type, int char_posn, int length, char *name)
 	// This could be used to return an index to the word currently being spoken
 	// Type 7=phoneme
 
-	int *p;
-
 	if (WcmdqFree() > 5) {
 		wcmdq[wcmdq_tail][0] = WCMD_MARKER + (type << 8);
 		wcmdq[wcmdq_tail][1] = (char_posn & 0xffffff) | (length << 24);
-		p = (int *)name;
-		wcmdq[wcmdq_tail][2] = p[0]; // up to 8 bytes of UTF8 characters
-		wcmdq[wcmdq_tail][3] = p[1];
+		memcpy(&wcmdq[wcmdq_tail][2], name, 8); // up to 8 bytes of UTF8 characters
 		WcmdqInc();
 	}
 }
 
-#if HAVE_SONIC_H
+#if USE_LIBSONIC
 void DoSonicSpeed(int value)
 {
 	// value, multiplier * 1024
@@ -1077,10 +1067,11 @@ void DoEmbedded(int *embix, int sourceix)
 {
 	// There were embedded commands in the text at this point
 	unsigned int word; // bit 7=last command for this word, bits 5,6 sign, bits 0-4 command
-	unsigned int value;
-	int command;
 
 	do {
+		unsigned int value;
+		int command;
+
 		word = embedded_list[*embix];
 		value = word >> 8;
 		command = word & 0x7f;
@@ -1132,9 +1123,6 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, bool resume)
 	static int ix;
 	static int embedded_ix;
 	static int word_count;
-	PHONEME_LIST *prev;
-	PHONEME_LIST *next;
-	PHONEME_LIST *next2;
 	PHONEME_LIST *p;
 	bool released;
 	int stress;
@@ -1142,11 +1130,10 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, bool resume)
 	bool  pre_voiced;
 	int free_min;
 	int value;
-	unsigned char *pitch_env = NULL;
-	unsigned char *amp_env;
+	const unsigned char *pitch_env = NULL;
+	const unsigned char *amp_env;
 	PHONEME_TAB *ph;
 	int use_ipa = 0;
-	bool done_phoneme_marker;
 	int vowelstart_prev;
 	char phoneme_name[16];
 	static int sourceix = 0;
@@ -1161,8 +1148,10 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, bool resume)
 	if (option_phoneme_events & espeakINITIALIZE_PHONEME_IPA)
 		use_ipa = 1;
 
+#if USE_MBROLA
 	if (mbrola_name[0] != 0)
 		return MbrolaGenerate(phoneme_list, n_ph, resume);
+#endif
 
 	if (resume == false) {
 		ix = 1;
@@ -1182,12 +1171,14 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, bool resume)
 
 	while ((ix < (*n_ph)) && (ix < N_PHONEME_LIST-2)) {
 		p = &phoneme_list[ix];
-		
+
 		if(output_hooks && output_hooks->outputPhoSymbol)
 		{
 			char buf[30];
 			int dummy=0;
-			WritePhMnemonic(buf, p->ph, p, 0, &dummy);
+			//WritePhMnemonic(buf, p->ph, p, 0, &dummy);
+			WritePhMnemonicWithStress(buf, p->ph, p, 0, &dummy);
+
 			DoPhonemeAlignment(strdup(buf),p->type);
 		}
 
@@ -1201,6 +1192,10 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, bool resume)
 		if (WcmdqFree() <= free_min)
 			return 1; // wait
 
+		PHONEME_LIST *prev;
+		PHONEME_LIST *next;
+		PHONEME_LIST *next2;
+
 		prev = &phoneme_list[ix-1];
 		next = &phoneme_list[ix+1];
 		next2 = &phoneme_list[ix+2];
@@ -1228,12 +1223,14 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, bool resume)
 		if ((p->prepause > 0) && !(p->ph->phflags & phPREVOICE))
 			DoPause(p->prepause, 1);
 
-		done_phoneme_marker = false;
+		bool done_phoneme_marker = false;
 		if (option_phoneme_events && (p->ph->code != phonEND_WORD)) {
 			if ((p->type == phVOWEL) && (prev->type == phLIQUID || prev->type == phNASAL)) {
 				// For vowels following a liquid or nasal, do the phoneme event after the vowel-start
 			} else {
-				WritePhMnemonic(phoneme_name, p->ph, p, use_ipa, NULL);
+				//WritePhMnemonic(phoneme_name, p->ph, p, use_ipa, NULL);
+				WritePhMnemonicWithStress(phoneme_name, p->ph, p, use_ipa, NULL);
+
 				DoPhonemeMarker(espeakEVENT_PHONEME, sourceix, 0, phoneme_name);
 				done_phoneme_marker = true;
 			}
@@ -1497,7 +1494,9 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, bool resume)
 			}
 
 			if ((option_phoneme_events) && (done_phoneme_marker == false)) {
-				WritePhMnemonic(phoneme_name, p->ph, p, use_ipa, NULL);
+				//WritePhMnemonic(phoneme_name, p->ph, p, use_ipa, NULL);
+				WritePhMnemonicWithStress(phoneme_name, p->ph, p, use_ipa, NULL);
+
 				DoPhonemeMarker(espeakEVENT_PHONEME, sourceix, 0, phoneme_name);
 			}
 
@@ -1546,7 +1545,6 @@ int SpeakNextClause(int control)
 
 	int clause_tone;
 	char *voice_change;
-	const char *phon_out;
 
 	if (control == 2) {
 		// stop speaking
@@ -1561,8 +1559,7 @@ int SpeakNextClause(int control)
 		return 0;
 	}
 
-	if (current_phoneme_table != voice->phoneme_tab_ix)
-		SelectPhonemeTable(voice->phoneme_tab_ix);
+	SelectPhonemeTable(voice->phoneme_tab_ix);
 
 	// read the next clause from the input text file, translate it, and generate
 	// entries in the wavegen command queue
@@ -1572,6 +1569,7 @@ int SpeakNextClause(int control)
 	CalcLengths(translator);
 
 	if ((option_phonemes & 0xf) || (phoneme_callback != NULL)) {
+		const char *phon_out;
 		phon_out = GetTranslatedPhonemeString(option_phonemes);
 		if (option_phonemes & 0xf)
 			fprintf(f_trans, "%s\n", phon_out);
@@ -1600,3 +1598,10 @@ int SpeakNextClause(int control)
 
 	return 1;
 }
+
+#pragma GCC visibility push(default)
+ESPEAK_API void espeak_SetPhonemeCallback(int (*PhonemeCallback)(const char *))
+{
+	phoneme_callback = PhonemeCallback;
+}
+#pragma GCC visibility pop
