@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2005 to 2014 by Jonathan Duddington
  * email: jonsd@users.sourceforge.net
- * Copyright (C) 2015 Reece H. Dunn
+ * Copyright (C) 2015-2017, 2020 Reece H. Dunn
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,18 +17,29 @@
  * along with this program; if not, see: <http://www.gnu.org/licenses/>.
  */
 
+#ifndef ESPEAK_NG_TRANSLATE_H
+#define ESPEAK_NG_TRANSLATE_H
+
+#include <stdbool.h>
+
+#include <espeak-ng/espeak_ng.h>
+#include <espeak-ng/encoding.h>
+
 #ifdef __cplusplus
 extern "C"
 {
 #endif
 
 #define L(c1, c2) (c1<<8)+c2 // combine two characters into an integer for translator name
+#define L3(c1, c2, c3) (c1<<16)+(c2<<8) + c3 // combine three characters into an integer for translator name
+#define L4(c1, c2, c3, c4) (c1<<24)+(c2<<16)+(c3<<8) + c4 // combine four characters into an integer for translator name
 
 #define CTRL_EMBEDDED    0x01 // control character at the start of an embedded command
 #define REPLACED_E       'E' // 'e' replaced by silent e
 
 #define N_WORD_PHONEMES  200 // max phonemes in a word
 #define N_WORD_BYTES     160 // max bytes for the UTF8 characters in a word
+#define N_PHONEME_BYTES  160 // max bytes for a phoneme
 #define N_CLAUSE_WORDS   300 // max words in a clause
 #define N_TR_SOURCE      800 // the source text of a single clause (UTF8 bytes)
 
@@ -53,9 +64,6 @@ extern "C"
 #define FLAG_ALT_TRANS      0x8000 // language specific
 #define FLAG_ALT2_TRANS    0x10000 // language specific
 #define FLAG_ALT3_TRANS    0x20000 // language specific
-#define FLAG_ALT4_TRANS    0x40000 // language specific
-#define FLAG_ALT5_TRANS    0x80000 // language specific
-#define FLAG_ALT6_TRANS   0x100000 // language specific
 #define FLAG_ALT7_TRANS   0x200000 // language specific
 
 #define FLAG_COMBINE      0x800000 // combine with the next word
@@ -82,7 +90,6 @@ extern "C"
 #define FLAG_CAPITAL         0x200 // pronunciation if initial letter is upper case
 #define FLAG_ALLCAPS         0x400 // only if the word is all capitals
 #define FLAG_ACCENT          0x800 // character name is base-character name + accent name
-#define FLAG_HYPHENATED     0x1000 // multiple-words, but needs hyphen between parts 1 and 2
 #define FLAG_SENTENCE       0x2000 // only if the clause is a sentence
 #define FLAG_ONLY           0x4000
 #define FLAG_ONLY_S         0x8000
@@ -106,7 +113,7 @@ extern "C"
 #define FLAG_LAST_WORD     0x10  // last word in clause
 #define FLAG_EMBEDDED      0x40  // word is preceded by embedded commands
 #define FLAG_HYPHEN        0x80
-#define FLAG_NOSPACE       0x100 // word is not seperated from previous word by a space
+#define FLAG_NOSPACE       0x100 // word is not separated from previous word by a space
 #define FLAG_FIRST_WORD    0x200 // first word in clause
 #define FLAG_FOCUS         0x400 // the focus word of a clause
 #define FLAG_EMPHASIZED    0x800
@@ -235,8 +242,6 @@ extern "C"
 #define CLAUSE_COLON       (30 | CLAUSE_INTONATION_FULL_STOP   | CLAUSE_TYPE_CLAUSE)
 #define CLAUSE_SEMICOLON   (30 | CLAUSE_INTONATION_COMMA       | CLAUSE_TYPE_CLAUSE)
 
-int clause_type_from_codepoint(uint32_t c);
-
 //@}
 
 #define SAYAS_CHARS        0x12
@@ -254,43 +259,23 @@ int clause_type_from_codepoint(uint32_t c);
 //     match 1 pre 2 post 0     - use common phoneme string
 //     match 1 pre 2 post 3 0   - empty phoneme string
 
-typedef const char *constcharptr;
-
-typedef struct {
-	int points;
-	const char *phonemes;
-	int end_type;
-	char *del_fwd;
-} MatchRecord;
-
 // used to mark words with the source[] buffer
 typedef struct {
 	unsigned int flags;
 	unsigned short start;
 	unsigned char pre_pause;
-	unsigned char wmark;
 	unsigned short sourceix;
 	unsigned char length;
 } WORD_TAB;
 
-typedef struct {
-	int type;
-	int parameter[N_SPEECH_PARAM];
-} PARAM_STACK;
-
-extern PARAM_STACK param_stack[];
-extern const int param_defaults[N_SPEECH_PARAM];
-
 typedef struct {
 	const char *name;
 	int offset;
-	unsigned short range_min, range_max;
+	unsigned int range_min, range_max;
 	int language;
 	int flags;
 } ALPHABET;
 
-extern ALPHABET alphabets[];
-extern ALPHABET *current_alphabet;
 // alphabet flags
 #define AL_DONT_NAME    0x01 // don't speak the alphabet name
 #define AL_NOT_LETTERS  0x02 // don't use the language for speaking letters
@@ -298,79 +283,72 @@ extern ALPHABET *current_alphabet;
 #define AL_NOT_CODE     0x08 // don't speak the character code
 #define AL_NO_SYMBOL    0x10 // don't repeat "symbol" or "character"
 
-#define N_LOPTS       21
-#define LOPT_DIERESES  1
+#define N_LOPTS       18
+#define LOPT_DIERESES  0
 // 1=remove [:] from unstressed syllables, 2= remove from unstressed or non-penultimate syllables
 // bit 4=0, if stress < 4,  bit 4=1, if not the highest stress in the word
-#define LOPT_IT_LENGTHEN 2
+#define LOPT_IT_LENGTHEN 1
 
 // 1=german
-#define LOPT_PREFIXES 3
+#define LOPT_PREFIXES 2
 
 // non-zero, change voiced/unoiced to match last consonant in a cluster
 // bit 0=use regressive voicing
 // bit 1=LANG=cz,bg  don't propagate over [v]
 // bit 2=don't propagate acress word boundaries
 // bit 3=LANG=pl,  propagate over liquids and nasals
-// bit 4=LANG=cz,sk  don't progagate to [v]
+// bit 4=LANG=cz,sk  don't propagate to [v]
 // bit 8=devoice word-final consonants
-#define LOPT_REGRESSIVE_VOICING 4
+#define LOPT_REGRESSIVE_VOICING 3
 
 // 0=default, 1=no check, other allow this character as an extra initial letter (default is 's')
-#define LOPT_UNPRONOUNCABLE 5
-
-// select length_mods tables,  (length_mod_tab) + (length_mod_tab0 * 100)
-#define LOPT_LENGTH_MODS 6
+#define LOPT_UNPRONOUNCABLE 4
 
 // increase this to prevent sonorants being shortened before shortened (eg. unstressed) vowels
-#define LOPT_SONORANT_MIN 7
+#define LOPT_SONORANT_MIN 5
 
 // bit 0: don't break vowels at word boundary
-#define LOPT_WORD_MERGE 8
+#define LOPT_WORD_MERGE 6
+
 
 // max. amplitude for vowel at the end of a clause
-#define LOPT_MAXAMP_EOC 9
+#define LOPT_MAXAMP_EOC 7
 
 // bit 0=reduce even if phonemes are specified in the **_list file
 // bit 1=don't reduce the strongest vowel in a word which is marked 'unstressed'
-#define LOPT_REDUCE 10
+#define LOPT_REDUCE 8
 
 // LANG=cs,sk  combine some prepositions with the following word, if the combination has N or fewer syllables
 // bits 0-3  N syllables
 // bit 4=only if the second word has $alt attribute
 // bit 5=not if the second word is end-of-sentence
-#define LOPT_COMBINE_WORDS 11
+#define LOPT_COMBINE_WORDS 9
 
-// change [t] when followed by unstressed vowel
-#define LOPT_REDUCE_T 12
-
-// 1 = allow capitals inside a word
-// 2 = stressed syllable is indicated by capitals
-#define LOPT_CAPS_IN_WORD 13
-
-// bit 0=Italian "syntactic doubling" of consoants in the word after a word marked with $double attribute
-// bit 1=also after a word which ends with a stressed vowel
-#define LOPT_IT_DOUBLING 14
+// 1 = stressed syllable is indicated by capitals
+#define LOPT_CAPS_IN_WORD 10
 
 // Call ApplySpecialAttributes() if $alt or $alt2 is set for a word
 // bit 1: stressed syllable: $alt change [e],[o] to [E],[O],  $alt2 change [E],[O] to [e],[o]
-#define LOPT_ALT 15
+#define LOPT_ALT 11
 
-// pause for bracket (default=4), pause when annoucing bracket names (default=2)
-#define LOPT_BRACKET_PAUSE 16
+// pause for bracket (default=4), also see LOPT_BRACKET_PAUSE_ANNOUNCED
+#define LOPT_BRACKET_PAUSE 12
 
 // bit 1, don't break clause before annoucning . ? !
-#define LOPT_ANNOUNCE_PUNCT 17
+#define LOPT_ANNOUNCE_PUNCT 13
 
 // recognize long vowels (0 = don't recognize)
-#define LOPT_LONG_VOWEL_THRESHOLD 18
+#define LOPT_LONG_VOWEL_THRESHOLD 14
 
 // bit 0:  Don't allow suffices if there is no previous syllable
-#define LOPT_SUFFIX 19
+#define LOPT_SUFFIX 15
 
 // bit 0  Apostrophe at start of word is part of the word
 // bit 1  Apostrophe at end of word is part of the word
-#define LOPT_APOSTROPHE 20
+#define LOPT_APOSTROPHE 16
+
+// pause when announcing bracket names (default=2), also see LOPT_BRACKET_PAUSE
+#define LOPT_BRACKET_PAUSE_ANNOUNCED 17
 
 // stress_rule
 #define STRESSPOSN_1L 0 // 1st syllable
@@ -378,6 +356,14 @@ extern ALPHABET *current_alphabet;
 #define STRESSPOSN_2R 2 // penultimate
 #define STRESSPOSN_1R 3 // final syllable
 #define STRESSPOSN_3R 4 // antipenultimate
+#define STRESSPOSN_SYLCOUNT 5 // stress depends on syllable count
+#define STRESSPOSN_1RH 6 // last heaviest syllable, excluding final syllable
+#define STRESSPOSN_1RU 7 // stress on the last syllable, before any explicitly unstressed syllable
+#define STRESSPOSN_2LLH 8 // first syllable, unless it is a light syllable followed by a heavy syllable
+#define STRESSPOSN_ALL 9 // mark all stressed
+#define STRESSPOSN_GREENLANDIC 12
+#define STRESSPOSN_1SL 13 // 1st syllable, unless 1st vowel is short and 2nd is long
+#define STRESSPOSN_EU 15 // If more than 2 syllables: primary stress in second syllable and secondary on last.
 
 typedef struct {
 // bits0-2  separate words with (1=pause_vshort, 2=pause_short, 3=pause, 4=pause_long 5=[?] phonemme)
@@ -386,7 +372,7 @@ typedef struct {
 // bit5=length of a final vowel doesn't depend on the next phoneme
 	int word_gap;
 	int vowel_pause;
-	int stress_rule; // 1=first syllable, 2=penultimate,  3=last
+	int stress_rule; // see #defines for STRESSPOSN_*
 
 #define S_NO_DIM            0x02
 #define S_FINAL_DIM         0x04
@@ -445,106 +431,84 @@ typedef struct {
 	int unstressed_wd1; // stress for $u word of 1 syllable
 	int unstressed_wd2; // stress for $u word of >1 syllable
 	int param[N_LOPTS];
-	int param2[N_LOPTS];
-	unsigned char *length_mods;
-	unsigned char *length_mods0;
+	const unsigned char *length_mods;
+	const unsigned char *length_mods0;
 
-#define NUM_THOUS_SPACE  0x4
-#define NUM_DECIMAL_COMMA 0x8
-#define NUM_SWAP_TENS    0x10
-#define NUM_AND_UNITS    0x20
-#define NUM_HUNDRED_AND  0x40
-#define NUM_SINGLE_AND   0x80
-#define NUM_SINGLE_STRESS 0x100
-#define NUM_SINGLE_VOWEL 0x200
-#define NUM_OMIT_1_HUNDRED 0x400
-#define NUM_1900         0x800
-#define NUM_ALLOW_SPACE  0x1000
-#define NUM_DFRACTION_1  0x2000
-#define NUM_DFRACTION_2  0x4000
-#define NUM_DFRACTION_3  0x6000
-#define NUM_DFRACTION_4  0x8000
-#define NUM_DFRACTION_5  0xa000
-#define NUM_DFRACTION_6  0xc000
-#define NUM_DFRACTION_7  0xe000    // lang=si, alternative form of number for decimal fraction digits (except the last)
-#define NUM_ORDINAL_DOT   0x10000
-#define NUM_NOPAUSE       0x20000
-#define NUM_AND_HUNDRED   0x40000
-#define NUM_THOUSAND_AND  0x80000
-#define NUM_VIGESIMAL       0x100000
-#define NUM_OMIT_1_THOUSAND 0x200000
-#define NUM_ZERO_HUNDRED    0x400000
-#define NUM_HUNDRED_AND_DIGIT   0x800000
-#define NUM_ROMAN          0x1000000
-#define NUM_ROMAN_CAPITALS 0x2000000
-#define NUM_ROMAN_AFTER    0x4000000
-#define NUM_ROMAN_ORDINAL  0x8000000
-#define NUM_SINGLE_STRESS_L  0x10000000
+#define NUM_DEFAULT           0x00000001 // enable number processing; use if no other NUM_ option is specified
+#define NUM_THOUS_SPACE       0x00000004 // thousands separator must be space
+#define NUM_DECIMAL_COMMA     0x00000008 // , decimal separator, not .
+#define NUM_SWAP_TENS         0x00000010 // use three-and-twenty rather than twenty-three
+#define NUM_AND_UNITS         0x00000020 // 'and' between tens and units
+#define NUM_HUNDRED_AND       0x00000040 // add "and" after hundred or thousand
+#define NUM_SINGLE_AND        0x00000080 // don't have "and" both after hundreds and also between tens and units
+#define NUM_SINGLE_STRESS     0x00000100 // only one primary stress in tens+units
+#define NUM_SINGLE_VOWEL      0x00000200 // only one vowel between tens and units
+#define NUM_OMIT_1_HUNDRED    0x00000400 // omit "one" before "hundred"
+#define NUM_1900              0x00000800 // say 19** as nineteen hundred
+#define NUM_ALLOW_SPACE       0x00001000 // allow space as thousands separator (in addition to langopts.thousands_sep)
+#define NUM_DFRACTION_BITS    0x0000e000 // post-decimal-digits 0=single digits, 1=(LANG=it) 2=(LANG=pl) 3=(LANG=ro)
+#define NUM_ORDINAL_DOT       0x00010000 // dot after number indicates ordinal
+#define NUM_NOPAUSE           0x00020000 // don't add pause after a number
+#define NUM_AND_HUNDRED       0x00040000 // 'and' before hundreds
+#define NUM_THOUSAND_AND      0x00080000 // 'and' after thousands if there are no hundreds
+#define NUM_VIGESIMAL         0x00100000 // vigesimal number, if tens are not found
+#define NUM_OMIT_1_THOUSAND   0x00200000 // omit "one" before "thousand"
+#define NUM_ZERO_HUNDRED      0x00400000 // say "zero" before hundred
+#define NUM_HUNDRED_AND_DIGIT 0x00800000 // add "and" after hundreds and thousands, only if there are digits and no tens
+#define NUM_ROMAN             0x01000000 // recognize roman numbers
+#define NUM_ROMAN_CAPITALS    0x02000000 // Roman numbers only if upper case
+#define NUM_ROMAN_AFTER       0x04000000 // say "roman" after the number, not before
+#define NUM_ROMAN_ORDINAL     0x08000000 // Roman numbers are ordinal numbers
+#define NUM_SINGLE_STRESS_L   0x10000000 // only one primary stress in tens+units (on the tens)
 
-	// bits0-1=which numbers routine to use.
-	// bit2=  thousands separator must be space
-	// bit3=  , decimal separator, not .
-	// bit4=use three-and-twenty rather than twenty-three
-	// bit5='and' between tens and units
-	// bit6=add "and" after hundred or thousand
-	// bit7=don't have "and" both after hundreds and also between tens and units
-	// bit8=only one primary stress in tens+units
-	// bit9=only one vowel betwen tens and units
-	// bit10=omit "one" before "hundred"
-	// bit11=say 19** as nineteen hundred
-	// bit12=allow space as thousands separator (in addition to langopts.thousands_sep)
-	// bits13-15  post-decimal-digits 0=single digits, 1=(LANG=it) 2=(LANG=pl) 3=(LANG=ro)
+#define NUM_DFRACTION_1       0x00002000
+#define NUM_DFRACTION_2       0x00004000
+#define NUM_DFRACTION_3       0x00006000
+#define NUM_DFRACTION_4       0x00008000
+#define NUM_DFRACTION_5       0x0000a000
+#define NUM_DFRACTION_6       0x0000c000
+#define NUM_DFRACTION_7       0x0000e000    // lang=si, alternative form of number for decimal fraction digits (except the last)
 
-	// bit16= dot after number indicates ordinal
-	// bit17= don't add pause after a number
-	// bit18= 'and' before hundreds
-	// bit19= 'and' after thousands if there are no hundreds
-	// bit20= vigesimal number, if tens are not found
-	// bit21= omit "one" before "thousand"
-	// bit22= say "zero" before hundred
-	// bit23= add "and" after hundreds and thousands, only if there are digits and no tens
-
-	// bit24= recognize roman numbers
-	// bit25= Roman numbers only if upper case
-	// bit26= say "roman" after the number, not before
-	// bit27= Roman numbers are ordinal numbers
-	// bit28= only one primary stress in tens+units (on the tens)
 	int numbers;
 
-#define NUM2_THOUSANDS_VAR1     0x40
-#define NUM2_THOUSANDS_VAR2     0x80
-#define NUM2_THOUSANDS_VAR3     0xc0
-#define NUM2_THOUSANDS_VAR4     0x100
-#define NUM2_THOUSANDS_VAR5     0x140
+#define NUM2_THOUSANDS_VAR_BITS    0x000001c0 // use different forms of thousand, million, etc (M MA MB)
+#define NUM2_SWAP_THOUSANDS        0x00000200 // say "thousand" and "million" before its number, not after
+#define NUM2_ORDINAL_NO_AND        0x00000800 // don't say 'and' between tens and units for ordinal numbers
+#define NUM2_MULTIPLE_ORDINAL      0x00001000 // use ordinal form of hundreds and tens as well as units
+#define NUM2_NO_TEEN_ORDINALS      0x00002000 // don't use 11-19 numbers to make ordinals
+#define NUM2_MYRIADS               0x00004000 // use myriads (groups of 4 digits) not thousands (groups of 3)
+#define NUM2_ENGLISH_NUMERALS      0x00008000 // speak (non-replaced) English numerals in English
+#define NUM2_PERCENT_BEFORE        0x00010000 // say "%" before the number
+#define NUM2_OMIT_1_HUNDRED_ONLY   0x00020000 // omit "one" before hundred only if there are no previous digits
+#define NUM2_ORDINAL_AND_THOUSANDS 0x00040000 // same variant for ordinals and thousands (#o = #a)
+#define NUM2_ORDINAL_DROP_VOWEL    0x00080000 // drop final vowel from cardial number before adding ordinal suffix (currently only tens and units)
+#define NUM2_ZERO_TENS             0x00100000 // say zero tens
+
+#define NUM2_THOUSANDPLEX_VAR_THOUSANDS 0x00000002
+#define NUM2_THOUSANDPLEX_VAR_MILLIARDS 0x00000008
+#define NUM2_THOUSANDPLEX_VAR_ALL       0x0000001e
+
+#define NUM2_THOUSANDS_VAR1        0x00000040
+#define NUM2_THOUSANDS_VAR2        0x00000080
+#define NUM2_THOUSANDS_VAR3        0x000000c0
+#define NUM2_THOUSANDS_VAR4        0x00000100 // plural forms for millions, etc.
+#define NUM2_THOUSANDS_VAR5        0x00000140
 
-#define NUM2_SWAP_THOUSANDS     0x200
-#define NUM2_ORDINAL_NO_AND     0x800
-#define NUM2_MULTIPLE_ORDINAL   0x1000
-#define NUM2_NO_TEEN_ORDINALS   0x2000
-#define NUM2_MYRIADS            0x4000
-#define NUM2_ENGLISH_NUMERALS   0x8000
-#define NUM2_PERCENT_BEFORE     0x10000
-#define NUM2_OMIT_1_HUNDRED_ONLY 0x20000
-#define NUM2_ORDINAL_AND_THOUSANDS 0x40000
-#define NUM2_ORDINAL_DROP_VOWEL  0x80000 // currently only for tens and units
-#define NUM2_ZERO_TENS          0x100000
-	// bits 1-4  use variant form of numbers before thousands,millions,etc.
-	// bits 6-8  use different forms of thousand, million, etc (M MA MB)
-	// bit9=(LANG=rw) say "thousand" and "million" before its number, not after
-	// bit11=(LANG=es,an) don't say 'and' between tens and units for ordinal numbers
-	// bit12=(LANG=el,es) use ordinal form of hundreds and tens as well as units
-	// bit13=(LANG=pt) don't use 11-19 numbers to make ordinals
-	// bit14=(LANG=ko)  use myriads (groups of 4 digits) not thousands (groups of 3)
-	// bit15=(LANG=ne)  speak (non-replaced) English numerals in English
-	// bit16=(LANG=si)  say "%" before the number
-	// bit17=(LANG=ml)  omit "one" before hundred only if there are no previous digits
-	// bit18=(LANG=ta)  same variant for ordinals and thousands (#o = #a)
-	// bit19=(LANG=te)  drop final vowel from cardial number before adding ordinal suffix
-	// bit20=(LANG=zh)  say zero tens
 	int numbers2;
 
-#define BREAK_THOUSANDS   0x49249248
-	int break_numbers;  // which digits to break the number into thousands, millions, etc (Hindi has 100,000 not 1,000,000)
+// Bit 2^n is set if 10^n separates a number grouping (max n=31).
+//                                      0         1         2         3
+//                                  n = 01234567890123456789012345678901
+#define BREAK_THOUSANDS   0x49249248 // b  b  b  b  b  b  b  b  b  b  b  // 10,000,000,000,000,000,000,000,000,000,000
+#define BREAK_MYRIADS     0x11111110 // b   b   b   b   b   b   b   b    // 1000,0000,0000,0000,0000,0000,0000,0000
+#define BREAK_LAKH        0xaaaaaaa8 // b  b b b b b b b b b b b b b b b // 10,00,00,00,00,00,00,00,00,00,00,00,00,00,00,000
+#define BREAK_LAKH_BN     0x24924aa8 // b  b b b b b  b  b  b  b  b  b   // 100,000,000,000,000,000,000,00,00,00,00,000
+#define BREAK_LAKH_DV     0x000014a8 // b  b b b  b b                    // 100,00,000,00,00,000
+#define BREAK_LAKH_HI     0x00014aa8 // b  b b b b b  b b                // 100,00,000,00,00,00,00,000
+#define BREAK_LAKH_UR     0x000052a8 // b  b b b b  b b                  // 100,00,000,00,00,00,000
+#define BREAK_INDIVIDUAL  0x00000018 // b  bb                            // 100,0,000
+
+	unsigned break_numbers;  // which digits to break the number into thousands, millions, etc (Hindi has 100,000 not 1,000,000)
 	int max_roman;
 	int min_roman;
 	int thousands_sep;
@@ -560,34 +524,23 @@ typedef struct {
 	int intonation_group;
 	unsigned char tunes[6];
 	int long_stop;          // extra mS pause for a lengthened stop
-	int phoneme_change;     // TEST, change phonemes, after translation
 	char max_initial_consonants;
-	char spelling_stress;   // 0=default, 1=stress first letter
+	bool spelling_stress;
 	char tone_numbers;
 	char ideographs;      // treat as separate words
-	char textmode;          // the meaning of FLAG_TEXTMODE is reversed (to save data when *_list file is compiled)
+	bool textmode;          // the meaning of FLAG_TEXTMODE is reversed (to save data when *_list file is compiled)
 	char dotless_i;         // uses letter U+0131
-	int testing;            // testing options: bit 1= specify stressed syllable in the form:  "outdoor/2"
 	int listx;    // compile *_listx after *list
-	const unsigned int *replace_chars;      // characters to be substitutes
+	const unsigned char *replace_chars;      // characters to be substitutes
 	int our_alphabet;           // offset for main alphabet (if not set in letter_bits_offset)
 	int alt_alphabet;       // offset for another language to recognize
 	int alt_alphabet_lang;  // language for the alt_alphabet
 	int max_lengthmod;
 	int lengthen_tonic;   // lengthen the tonic syllable
 	int suffix_add_e;      // replace a suffix (which has the SUFX_E flag) with this character
+	bool lowercase_sentence;	// when true, a period . causes a sentence stop even if next character is lowercase
 } LANGUAGE_OPTIONS;
 
-// a parameter of ChangePhonemes()
-typedef struct {
-	int flags;
-	unsigned char stress;          // stress level of this vowel
-	unsigned char stress_highest;  // the highest stress level of a vowel in this word
-	unsigned char n_vowels;        // number of vowels in the word
-	unsigned char vowel_this;      // syllable number of this vowel (counting from 1)
-	unsigned char vowel_stressed;  // syllable number of the highest stressed vowel
-} CHANGEPH;
-
 typedef struct {
 	LANGUAGE_OPTIONS langopts;
 	int translator_name;
@@ -601,7 +554,6 @@ typedef struct {
 	int phoneme_tab_ix;
 
 	unsigned char stress_amps[8];
-	unsigned char stress_amps_r[8];
 	short stress_lengths[8];
 	int dict_condition;    // conditional apply some pronunciation rules and dict.lookups
 	int dict_min_size;
@@ -655,9 +607,9 @@ typedef struct {
 	int end_stressed_vowel;  // word ends with stressed vowel
 	int prev_dict_flags[2];     // dictionary flags from previous word
 	int clause_terminator;
+
 } Translator;
 
-extern int option_tone2;
 #define OPTION_EMPHASIZE_ALLCAPS  0x100
 #define OPTION_EMPHASIZE_PENULTIMATE 0x200
 extern int option_tone_flags;
@@ -669,17 +621,15 @@ extern int option_punctuation;
 extern int option_endpause;
 extern int option_ssml;
 extern int option_phoneme_input;   // allow [[phonemes]] in input text
-extern int option_phoneme_variants;
 extern int option_sayas;
 extern int option_wordgap;
 
 extern int count_characters;
-extern int count_words;
 extern int count_sentences;
 extern int skip_characters;
 extern int skip_words;
 extern int skip_sentences;
-extern int skipping_text;
+extern bool skipping_text;
 extern int end_character_position;
 extern int clause_start_char;
 extern int clause_start_word;
@@ -691,86 +641,42 @@ extern char skip_marker[N_MARKER_LENGTH];
 
 #define N_PUNCTLIST  60
 extern wchar_t option_punctlist[N_PUNCTLIST];  // which punctuation characters to announce
-extern unsigned char punctuation_to_tone[INTONATION_TYPES][PUNCT_INTONATIONS];
 
 extern Translator *translator;
 extern Translator *translator2;
+extern Translator *translator3;
 extern char dictionary_name[40];
-extern char ctrl_embedded;    // to allow an alternative CTRL for embedded commands
 extern espeak_ng_TEXT_DECODER *p_decoder;
 extern int dictionary_skipwords;
 
-extern int (*uri_callback)(int, const char *, const char *);
-extern int (*phoneme_callback)(const char *);
-extern void SetLengthMods(Translator *tr, int value);
+#define LEADING_2_BITS 0xC0 // 0b11000000
+#define UTF8_TAIL_BITS 0x80 // 0b10000000
 
-void LoadConfig(void);
-int TransposeAlphabet(Translator *tr, char *text);
-ESPEAK_NG_API int utf8_in(int *c, const char *buf);
-int utf8_in2(int *c, const char *buf, int backwards);
-int utf8_out(unsigned int c, char *buf);
-int utf8_nbytes(const char *buf);
 int lookupwchar(const unsigned short *list, int c);
-int lookupwchar2(const unsigned short *list, int c);
-int Eof(void);
 char *strchr_w(const char *s, int c);
-int IsBracket(int c);
 void InitNamedata(void);
 void InitText(int flags);
 void InitText2(void);
-int IsDigit(unsigned int c);
-int IsDigit09(unsigned int c);
-int IsAlpha(unsigned int c);
-int IsVowel(Translator *tr, int c);
-int IsSuperscript(int letter);
-int isspace2(unsigned int c);
-int towlower2(unsigned int c); // Supports Turkish I
-const char *GetTranslatedPhonemeString(int phoneme_mode);
-const char *WordToString2(unsigned int word);
-ALPHABET *AlphabetFromChar(int c);
+const ALPHABET *AlphabetFromChar(int c);
 
 Translator *SelectTranslator(const char *name);
 int SetTranslator2(const char *name);
+int SetTranslator3(const char *name);
 void DeleteTranslator(Translator *tr);
 void ProcessLanguageOptions(LANGUAGE_OPTIONS *langopts);
-int Lookup(Translator *tr, const char *word, char *ph_out);
-int LookupFlags(Translator *tr, const char *word, unsigned int **flags_out);
 
-int TranslateNumber(Translator *tr, char *word1, char *ph_out, unsigned int *flags, WORD_TAB *wtab, int control);
-int TranslateRoman(Translator *tr, char *word, char *ph_out, WORD_TAB *wtab);
+void print_dictionary_flags(unsigned int *flags, char *buf, int buf_len);
 
-void ChangeWordStress(Translator *tr, char *word, int new_stress);
-void SetSpellingStress(Translator *tr, char *phonemes, int control, int n_chars);
-int TranslateLetter(Translator *tr, char *letter, char *phonemes, int control);
-void LookupLetter(Translator *tr, unsigned int letter, int next_byte, char *ph_buf, int control);
-void LookupAccentedLetter(Translator *tr, unsigned int letter, char *ph_buf);
-
-int LoadDictionary(Translator *tr, const char *name, int no_error);
-int LookupDictList(Translator *tr, char **wordptr, char *ph_out, unsigned int *flags, int end_flags, WORD_TAB *wtab);
-
-void MakePhonemeList(Translator *tr, int post_pause, int new_sentence);
-int ChangePhonemes_ru(Translator *tr, PHONEME_LIST2 *phlist, int n_ph, int index, PHONEME_TAB *ph, CHANGEPH *ch);
-void ApplySpecialAttribute2(Translator *tr, char *phonemes, int dict_flags);
-void AppendPhonemes(Translator *tr, char *string, int size, const char *ph);
-
-void CalcLengths(Translator *tr);
-void CalcPitches(Translator *tr, int clause_tone);
-
-int RemoveEnding(Translator *tr, char *word, int end_type, char *word_copy);
-int Unpronouncable(Translator *tr, char *word, int posn);
-void SetWordStress(Translator *tr, char *output, unsigned int *dictionary_flags, int tonic, int prev_stress);
-int TranslateRules(Translator *tr, char *p, char *phonemes, int size, char *end_phonemes, int end_flags, unsigned int *dict_flags);
 int TranslateWord(Translator *tr, char *word1, WORD_TAB *wtab, char *word_out);
 void TranslateClause(Translator *tr, int *tone, char **voice_change);
-int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_buf, int *tone_type, char *voice_change);
+void TranslateClauseWithTerminator(Translator *tr, int *tone_out, char **voice_change, int *terminator_out);
 
 void SetVoiceStack(espeak_VOICE *v, const char *variant_name);
-void InterpretPhoneme(Translator *tr, int control, PHONEME_LIST *plist, PHONEME_DATA *phdata, WORD_PH_DATA *worddata);
-void InterpretPhoneme2(int phcode, PHONEME_DATA *phdata);
-char *WritePhMnemonic(char *phon_out, PHONEME_TAB *ph, PHONEME_LIST *plist, int use_ipa, int *flags);
 
 extern FILE *f_trans; // for logging
 
 #ifdef __cplusplus
 }
 #endif
+
+#endif
